"""
nodal_averaged_creep_strains
============================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from mapdlOperatorsCore plugin, from "result" category
"""

class nodal_averaged_creep_strains(Operator):
    """Read nodal averaged creep strains as averaged nodal result from rst file.

      available inputs:
        - time_scoping (Scoping, list) (optional)
        - mesh_scoping (ScopingsContainer, Scoping, list) (optional)
        - fields_container (FieldsContainer) (optional)
        - streams_container (StreamsContainer, Stream) (optional)
        - data_sources (DataSources)
        - mesh (MeshedRegion) (optional)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.result.nodal_averaged_creep_strains()

      >>> # Make input connections
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
      >>> my_mesh_scoping = dpf.ScopingsContainer()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.result.nodal_averaged_creep_strains(time_scoping=my_time_scoping,mesh_scoping=my_mesh_scoping,fields_container=my_fields_container,streams_container=my_streams_container,data_sources=my_data_sources,mesh=my_mesh)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, time_scoping=None, mesh_scoping=None, fields_container=None, streams_container=None, data_sources=None, mesh=None, config=None, server=None):
        super().__init__(name="mapdl::rst::NCR", config = config, server = server)
        self._inputs = InputsNodalAveragedCreepStrains(self)
        self._outputs = OutputsNodalAveragedCreepStrains(self)
        if time_scoping !=None:
            self.inputs.time_scoping.connect(time_scoping)
        if mesh_scoping !=None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if streams_container !=None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources !=None:
            self.inputs.data_sources.connect(data_sources)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        spec = Specification(description="""Read nodal averaged creep strains as averaged nodal result from rst file.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "time_scoping", type_names=["scoping","vector<int32>"], optional=True, document=""""""), 
                                 1 : PinSpecification(name = "mesh_scoping", type_names=["scopings_container","scoping","vector<int32>"], optional=True, document=""""""), 
                                 2 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=True, document="""FieldsContainer already allocated modified inplace"""), 
                                 3 : PinSpecification(name = "streams_container", type_names=["streams_container","stream"], optional=True, document="""Streams containing the result file."""), 
                                 4 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document="""data sources containing the result file."""), 
                                 7 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=True, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""FieldsContainer filled in""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "mapdl::rst::NCR")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsNodalAveragedCreepStrains 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsNodalAveragedCreepStrains 
        """
        return super().outputs


#internal name: mapdl::rst::NCR
#scripting name: nodal_averaged_creep_strains
class InputsNodalAveragedCreepStrains(_Inputs):
    """Intermediate class used to connect user inputs to nodal_averaged_creep_strains operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.nodal_averaged_creep_strains()
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
      >>> my_mesh_scoping = dpf.ScopingsContainer()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
    """
    def __init__(self, op: Operator):
        super().__init__(nodal_averaged_creep_strains._spec().inputs, op)
        self._time_scoping = Input(nodal_averaged_creep_strains._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._time_scoping)
        self._mesh_scoping = Input(nodal_averaged_creep_strains._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh_scoping)
        self._fields_container = Input(nodal_averaged_creep_strains._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._fields_container)
        self._streams_container = Input(nodal_averaged_creep_strains._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._streams_container)
        self._data_sources = Input(nodal_averaged_creep_strains._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._data_sources)
        self._mesh = Input(nodal_averaged_creep_strains._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._mesh)

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator

        Parameters
        ----------
        my_time_scoping : Scoping, list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.nodal_averaged_creep_strains()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> #or
        >>> op.inputs.time_scoping(my_time_scoping)

        """
        return self._time_scoping

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator

        Parameters
        ----------
        my_mesh_scoping : ScopingsContainer, Scoping, list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.nodal_averaged_creep_strains()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> #or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)

        """
        return self._mesh_scoping

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        - pindoc: FieldsContainer already allocated modified inplace

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.nodal_averaged_creep_strains()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator

        - pindoc: Streams containing the result file.

        Parameters
        ----------
        my_streams_container : StreamsContainer, Stream, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.nodal_averaged_creep_strains()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> #or
        >>> op.inputs.streams_container(my_streams_container)

        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator

        - pindoc: data sources containing the result file.

        Parameters
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.nodal_averaged_creep_strains()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> #or
        >>> op.inputs.data_sources(my_data_sources)

        """
        return self._data_sources

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.nodal_averaged_creep_strains()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

class OutputsNodalAveragedCreepStrains(_Outputs):
    """Intermediate class used to get outputs from nodal_averaged_creep_strains operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.nodal_averaged_creep_strains()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(nodal_averaged_creep_strains._spec().outputs, op)
        self._fields_container = Output(nodal_averaged_creep_strains._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        - pindoc: FieldsContainer filled in

        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.nodal_averaged_creep_strains()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
set_property
============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "utility" category
"""

class set_property(Operator):
    """Set a property to an input field/field container

      available inputs:
        - field (Field, FieldsContainer)
        - property_name (str)
        - property_value (str, int, float)

      available outputs:
        - field (Field ,FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.utility.set_property()

      >>> # Make input connections
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_property_name = str()
      >>> op.inputs.property_name.connect(my_property_name)
      >>> my_property_value = str()
      >>> op.inputs.property_value.connect(my_property_value)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.utility.set_property(field=my_field,property_name=my_property_name,property_value=my_property_value)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, field=None, property_name=None, property_value=None, config=None, server=None):
        super().__init__(name="field::set_property", config = config, server = server)
        self._inputs = InputsSetProperty(self)
        self._outputs = OutputsSetProperty(self)
        if field !=None:
            self.inputs.field.connect(field)
        if property_name !=None:
            self.inputs.property_name.connect(property_name)
        if property_value !=None:
            self.inputs.property_value.connect(property_value)

    @staticmethod
    def _spec():
        spec = Specification(description="""Set a property to an input field/field container""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field","fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "property_name", type_names=["string"], optional=False, document="""Property to set"""), 
                                 2 : PinSpecification(name = "property_value", type_names=["string","int32","double"], optional=False, document="""Property to set""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field","fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "field::set_property")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsSetProperty 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsSetProperty 
        """
        return super().outputs


#internal name: field::set_property
#scripting name: set_property
class InputsSetProperty(_Inputs):
    """Intermediate class used to connect user inputs to set_property operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.utility.set_property()
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_property_name = str()
      >>> op.inputs.property_name.connect(my_property_name)
      >>> my_property_value = str()
      >>> op.inputs.property_value.connect(my_property_value)
    """
    def __init__(self, op: Operator):
        super().__init__(set_property._spec().inputs, op)
        self._field = Input(set_property._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field)
        self._property_name = Input(set_property._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._property_name)
        self._property_value = Input(set_property._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._property_value)

    @property
    def field(self):
        """Allows to connect field input to the operator

        Parameters
        ----------
        my_field : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.utility.set_property()
        >>> op.inputs.field.connect(my_field)
        >>> #or
        >>> op.inputs.field(my_field)

        """
        return self._field

    @property
    def property_name(self):
        """Allows to connect property_name input to the operator

        - pindoc: Property to set

        Parameters
        ----------
        my_property_name : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.utility.set_property()
        >>> op.inputs.property_name.connect(my_property_name)
        >>> #or
        >>> op.inputs.property_name(my_property_name)

        """
        return self._property_name

    @property
    def property_value(self):
        """Allows to connect property_value input to the operator

        - pindoc: Property to set

        Parameters
        ----------
        my_property_value : str, int, float, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.utility.set_property()
        >>> op.inputs.property_value.connect(my_property_value)
        >>> #or
        >>> op.inputs.property_value(my_property_value)

        """
        return self._property_value

class OutputsSetProperty(_Outputs):
    """Intermediate class used to get outputs from set_property operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.utility.set_property()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(set_property._spec().outputs, op)
        self.field_as_field = Output( _modify_output_spec_with_one_type(set_property._spec().output_pin(0), "field"), 0, op) 
        self._outputs.append(self.field_as_field)
        self.field_as_fields_container = Output( _modify_output_spec_with_one_type(set_property._spec().output_pin(0), "fields_container"), 0, op) 
        self._outputs.append(self.field_as_fields_container)




"""
FieldDefinition
================
"""

from ansys import dpf
from ansys.grpc.dpf import base_pb2, field_definition_pb2, field_definition_pb2_grpc
from ansys.dpf.core.common import natures, shell_layers
from ansys.dpf.core.dimensionality import Dimensionality


class FieldDefinition:
    """Contains the physical and mathematical description of the field.

    Parameters
    ----------
    field_definition : optional
        The default is ``None``.
    server : ansys.dpf.core.server, optional
        Server with the channel connected to the remote or local instance.
        The default is ``None``, in which case an attempt is made to use
        the global server.
    """

    def __init__(self, field_definition=None, server=None):
        if server is None:
            server = dpf.core._global_server()

        self._server = server
        self._stub = self._connect(self._server.channel)
        if isinstance(field_definition, field_definition_pb2.FieldDefinition):
            self._messageDefinition = field_definition
        else:
            request = base_pb2.Empty()
            self._messageDefinition = self._stub.Create(request)

    @property
    def location(self):
        """Field location.

        Returns
        -------
        str
            Location string, such as ``"Nodal"``, ``"Elemental"``,
            or ``"TimeFreq_sets"``.
        """
        out = self._stub.List(self._messageDefinition)
        return out.location.location

    @property
    def unit(self):
        """Units of the field.

        Returns
        -------
        str
            Units of the field.
        """
        return self._stub.List(self._messageDefinition).unit.symbol

    @property
    def shell_layers(self):
        """Order of the shell layers.

        Returns
        -------
        shell_layers : shell_layers
            ``LayerIndependent`` is returned for fields unrelated to layers.
        """
        enum_val = self._stub.List(self._messageDefinition).shell_layers
        return shell_layers(
            enum_val.real - 1
        )  # +1 is added to the proto enum to have notset as 0

    @property
    def dimensionality(self):
        """Dimensionality

        Returns
        -------
        dimensionality : Dimensionality
            Nature and size of the elementary data.
        """
        val = self._stub.List(
            self._messageDefinition
        ).dimensionnality  # typo exists on server side
        return Dimensionality(val.size, natures(val.nature.real))

    @unit.setter
    def unit(self, value):
        self._modify_field_def(unit=value)

    @location.setter
    def location(self, value):
        self._modify_field_def(location=value)

    @shell_layers.setter
    def shell_layers(self, value):
        self._modify_field_def(shell_layer=value)

    @dimensionality.setter
    def dimensionality(self, value):
        self._modify_field_def(dimensionality=value)

    def _modify_field_def(
        self, unit=None, location=None, dimensionality=None, shell_layer=None
    ):
        request = field_definition_pb2.FieldDefinitionUpdateRequest()
        request.field_definition.CopyFrom(self._messageDefinition)
        if unit != None:
            request.unit_symbol.symbol = unit
        if location != None:
            request.location.location = location
        if dimensionality != None:
            if not isinstance(dimensionality, Dimensionality):
                raise TypeError("the dimensionality needs to be of type Dimensionality")
            request.dimensionnality.CopyFrom(
                dimensionality._parse_dim_to_message()
            )  # typo is on server side
        if shell_layer != None:
            if isinstance(shell_layer, shell_layers):
                request.shell_layers = shell_layer.value + 1
            else:
                request.shell_layers = shell_layer + 1
        self._stub.Update(request)

    def __del__(self):
        try:
            self._stub.Delete(self._messageDefinition)
        except:
            pass

    def _connect(self, channel):
        """Connect to the gRPC service."""
        return field_definition_pb2_grpc.FieldDefinitionServiceStub(channel)



"""
.. _ref_operators_package:

ansys.dpf.core.operators
========================
"""
  

from . import result
from . import math
from . import min_max
from . import scoping
from . import utility
from . import metadata
from . import logic
from . import mesh
from . import filter
from . import serialization
from . import averaging
from . import geo
from . import invariant
from . import mapping



"""Miscellaneous functions for the DPF module."""
import platform
import glob
import os
from pkgutil import iter_modules


DEFAULT_FILE_CHUNK_SIZE = 65536

# ANSYS CPython Workbench environment may not have scooby installed.
try:
    from scooby import Report as ScoobyReport
except ImportError:

    class ScoobyReport:
        """Placeholder for Scooby report."""

        def __init__(self, *args, **kwargs):
            raise ImportError(
                "Install `scooby` with `pip install scooby` to use " "this feature"
            )


def module_exists(module_name):
    """Check if a model exists.

    Parameters
    ----------
    module_name : str
        Name of the module.

    """
    return module_name in (name for loader, name, ispkg in iter_modules())


class Report(ScoobyReport):
    """Generate a report of the installed packages for DPF-Core."""

    def __init__(self, additional=None, ncol=3, text_width=80, sort=False, gpu=True):
        """Generate a :class:`scooby.Report` instance.

        Parameters
        ----------
        additional : list(ModuleType), list(str)
            List of packages or package names to add to the output information.
        ncol : int, optional
            Number of package columns in the HTML table. This parameter has effect
            only if ``mode='HTML'`` or ``mode='html'``. The default is ``3``.
        text_width : int, optional
            Width of the text for non-HTML display modes. The default is ``80``.
        sort : bool, optional
            Whether to sort the packages alphabetically. The default is "False``.
        gpu : bool, optional
            Whether to gather information about the GPU. The default is ``True``.
            If rendering issues are experiencd, set to ``False`` to safely generate
            the report.

        """

        # Mandatory packages.
        core = ["ansys.grpc.dpf"]

        # Optional packages.
        optional = []

        # Information about the GPU - bare except in case there is a rendering
        # bug that the user is trying to report.
        if gpu:
            from pyvista.utilities.errors import GPUInfo

            try:
                extra_meta = [(t[1], t[0]) for t in GPUInfo().get_info()]
            except:
                extra_meta = ("GPU Details", "error")
        else:
            extra_meta = ("GPU Details", "None")

        super().__init__(
            additional=additional,
            core=core,
            optional=optional,
            ncol=ncol,
            text_width=text_width,
            sort=sort,
            extra_meta=extra_meta,
        )


def is_float(string):
    """Check if a string can be converted to a float.

    Parameters
    ----------
    string : str
       String to check.

    Returns
    -------
    bool
        ``True`` if the input can be converted to a float, ``False`` if otherwise.
    """
    try:
        float(string)
        return True
    except ValueError:
        return False


def is_ubuntu():
    """Check if the operating system is Ubuntu.

    Returns
    -------
    bool
        ``True`` when the operating system is Ubuntu, ``False`` if otherwise.
    """
    if os.name == "posix":
        return "ubuntu" in platform.platform().lower()
    return False


def find_ansys():
    """Search the standard installation location to find the path to the latest Ansys installation.

    Returns
    -------
    ansys_path : str
        Full path to the latest version of the Ansys installation.

    Examples
    --------
    Return path of latest Ansys version on Windows.

    >>> from ansys.dpf.core.misc import find_ansys
    >>> path = find_ansys()

    Return path of latest Ansys version on Linux.

    >>> path = find_ansys()

    """
    base_path = None
    if os.name == "nt":
        base_path = os.path.join(os.environ["PROGRAMFILES"], "ANSYS INC")
    elif os.name == "posix":
        for path in ["/usr/ansys_inc", "/ansys_inc"]:
            if os.path.isdir(path):
                base_path = path
    else:
        raise OSError(f"Unsupported OS {os.name}")

    if base_path is None:
        return base_path

    paths = glob.glob(os.path.join(base_path, "v*"))

    if not paths:
        return None

    versions = {}
    for path in paths:
        ver_str = path[-3:]
        if is_float(ver_str):
            versions[int(ver_str)] = path

    return versions[max(versions.keys())]



"""
.. _ref_field:

Field
=====
"""

from ansys import dpf
from ansys.dpf.core import errors, meshed_region, time_freq_support
from ansys.dpf.core.common import locations, natures, types
from ansys.dpf.core.field_base import _FieldBase, _LocalFieldBase
from ansys.dpf.core.field_definition import FieldDefinition
from ansys.dpf.core.plotter import Plotter
from ansys.grpc.dpf import base_pb2, field_pb2


class Field(_FieldBase):
    """Represents the main simulation data container.

    This can be evaluated data from the :class:`Operator <ansys.dpf.core.Operator>` class
    or created by a factory and directly by an instance of this class.

    A field's data is always associated to its scoping (entities
    associated to each value) and support (subset of the model where the
    data is), making the field a self-describing piece of data.

    Parameters
    ----------
    nentities : int, optional
        Number of entities reserved. The default is ``0``.
    nature : :class:`ansys.dpf.core.common.natures`, optional
        Nature of the field.
    location : str, optional
        Location of the field.  Options are:

        - ``"Nodal"``
        - ``"Elemental"``
        - ``"ElementalNodal"``

    field : ansys.grpc.dpf.field_pb2.Field, optional
        Field message generated from a gRPC stub.
    server : :class:`ansys.dpf.core.server`, optional
        Server with the channel connected to the remote or local instance. The
        default is ``None``, in which case an attempt is made to use the global
        server.
    Examples
    --------
    Create a field from scratch.

    >>> from ansys.dpf.core import fields_factory
    >>> from ansys.dpf.core import locations
    >>> from ansys.dpf import core as dpf
    >>> field_with_classic_api = dpf.Field()
    >>> field_with_classic_api.location = locations.nodal
    >>> field_with_factory = fields_factory.create_scalar_field(10)

    Extract a displacement field from a transient result file.

    >>> from ansys.dpf import core as dpf
    >>> from ansys.dpf.core import examples
    >>> transient = examples.download_transient_result()
    >>> model = dpf.Model(transient)
    >>> disp = model.results.displacement()
    >>> fields_container = disp.outputs.fields_container()
    >>> field = fields_container[0]
    >>> len(field)
    11460
    >>> field.component_count
    3
    >>> field.elementary_data_count
    3820

    Create a displacement field.

    >>> from ansys.dpf import core as dpf
    >>> import numpy as np
    >>> my_field = dpf.Field(10, dpf.natures.vector,dpf.locations.nodal)
    >>> my_field.data = np.zeros(30)
    >>> my_field.scoping.ids = range(1,11)

    Set data.

    >>> from ansys.dpf import core as dpf
    >>> from ansys.dpf.core import examples
    >>> transient = examples.download_transient_result()
    >>> model = dpf.Model(transient)
    >>> disp = model.results.displacement()
    >>> fields_container = disp.outputs.fields_container()
    >>> field = fields_container[0]
    >>> field.data[2]
    array([-0.00672665, -0.03213735,  0.00016716])

    """

    def __init__(
        self,
        nentities=0,
        nature=natures.vector,
        location=locations.nodal,
        field=None,
        server=None,
    ):
        """Initialize the field either with an optional field message or
        by connecting to a stub.
        """
        super().__init__(nentities, nature, location, False, field, server)
        self._field_definition = self._load_field_definition()

    def as_local_field(self):
        """Create a deep copy of the field that can be accessed and modified locally.

        This method allows you to access and modify the local copy of the field
        without sending a request to the server. It should be used in a ``with``
        statement so that the local field is released and the data is sent to
        the server in one action. If it is not used in a ``with`` statement,
        :func:`<release_data> Field.release_data()` should be used to update the field.

        Warning
        -------
        If this `as_local_field` method is not used as a context manager in a
        ``with`` statement or if the method `release_data()` is not called,
        the data will not be updated.

        Returns
        -------
        local_field : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> num_entities = 3
        >>> field_to_local = dpf.fields_factory.create_3d_vector_field(num_entities, location=dpf.locations.elemental_nodal)
        >>> with field_to_local.as_local_field() as f:
        ...     for i in range(1,num_entities+1):
        ...         f.append([[0.1*i,0.2*i, 0.3*i],[0.1*i,0.2*i, 0.3*i]],i)
        ...         f.get_entity_data(i-1),[[0.1*i,0.2*i, 0.3*i],[0.1*i,0.2*i, 0.3*i]]
        (array([[0.1, 0.2, 0.3],
               [0.1, 0.2, 0.3]]), [[0.1, 0.2, 0.3], [0.1, 0.2, 0.3]])
        (array([[0.2, 0.4, 0.6],
               [0.2, 0.4, 0.6]]), [[0.2, 0.4, 0.6], [0.2, 0.4, 0.6]])
        (array([[0.3, 0.6, 0.9],
               [0.3, 0.6, 0.9]]), [[0.30000000000000004, 0.6000000000000001, 0.8999999999999999], [0.30000000000000004, 0.6000000000000001, 0.8999999999999999]])

        """  # noqa: E501
        return _LocalField(self)

    @property
    def location(self):
        """Field location.

        Returns
        -------
        str
            Location string, which is ``"Nodal"``, ``"Elemental"``,
            or ``"ElementalNodal"``.

        Examples
        --------
        Location for a stress field evaluated at nodes.

        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.download_transient_result())
        >>> s_op = model.results.stress()
        >>> s_fc = s_op.outputs.fields_container()
        >>> field = s_fc[0]
        >>> field.location
        'ElementalNodal'

        """
        if self.field_definition:
            return self.field_definition.location

    @location.setter
    def location(self, value):
        """Change the field location.

        Parameters
        -------
        location : str or locations
            Location string, which is ``"Nodal"``, ``"Elemental"``,
            or ``"ElementalNodal"``.

        Examples
        --------
        Location for a field evaluated at nodes.

        >>> from ansys.dpf import core as dpf
        >>> import numpy as np
        >>> my_field = dpf.Field(10, dpf.natures.vector,dpf.locations.nodal)
        >>> my_field.data = np.zeros(30)
        >>> my_field.scoping.ids = range(1,11)
        >>> my_field.location
        'Nodal'
        >>> my_field.location = dpf.locations.elemental_nodal
        >>> my_field.location
        'ElementalNodal'

        """
        fielddef = self.field_definition
        fielddef.location = value
        self.field_definition = fielddef

    @property
    def shell_layers(self):
        """Order of the shell layers.

        Returns
        -------
        :class:`ansys.dpf.core.common.shell_layers`

        """
        if self.field_definition:
            return self.field_definition.shell_layers

    @shell_layers.setter
    def shell_layers(self, value):
        fielddef = self.field_definition
        fielddef.shell_layers = value
        self.field_definition = fielddef

    def to_nodal(self):
        """Convert the field to one with a ``Nodal`` location.

        This method is valid only when the field's current location is
        ``ElementalNodal`` or ``Elemental``.

        Returns
        -------
        nodal_field : Field
            with ``location=='Nodal'``.
        """
        if self.location == "Nodal":
            raise errors.LocationError('Location is already "Nodal"')

        op = dpf.core.Operator("to_nodal")
        op.inputs.connect(self)
        return op.outputs.field()

    def plot(self, notebook=None, shell_layers=None):
        """Plot the field or fields container on the mesh support if it exists.

        Warning
        -------
        This method is primarily added out of convenience as plotting
        directly from the field can be slower than extracting the
        meshed region and plotting the field on top of that.  It is
        more efficient to plot with:

        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> mesh = model.metadata.meshed_region
        >>> disp = model.results.displacement()
        >>> fields_container = disp.outputs.fields_container()
        >>> field = fields_container[0]
        >>> mesh.plot(field)

        Parameters
        ----------
        notebook : bool, optional
            Whether the plotting is in the notebook as
            a static image or is a dynamic plot outside of the
            notebook. The default is ``None``.
        shell_layers : shell_layers, optional
            Enum used to set the shell layers if the model to plot
            contains shell elements. The default is ``None``.
        """
        pl = Plotter(self.meshed_region)
        pl.plot_contour(self, notebook, shell_layers)

    def resize(self, nentities, datasize):
        """Allocate memory.

        Parameters
        ----------
        nentities : int
            Number of IDs in the scoping.
        datasize : int
            Size of the data vector.

        """
        request = field_pb2.UpdateSizeRequest()
        request.field.CopyFrom(self._message)
        request.size.scoping_size = nentities
        request.size.data_size = datasize
        self._stub.UpdateSize(request)

    def _load_field_definition(self):
        """Attempt to load the field definition for this field."""
        try:
            request = field_pb2.GetRequest()
            request.field.CopyFrom(self._message)
            out = self._stub.GetFieldDefinition(request)
            return FieldDefinition(out.field_definition, self._server)
        except:
            return

    @property
    def unit(self):
        """Units for the field.

        Returns
        ----------
        str
           Units for the field.

        Examples
        --------
        Units for a displacement field.

        >>> from ansys.dpf import core as dpf
        >>> my_field = dpf.Field(10, dpf.natures.vector,dpf.locations.nodal)
        >>> my_field.unit = "m"
        >>> my_field.unit
        'm'

        """
        if self.field_definition:
            return self.field_definition.unit

    @unit.setter
    def unit(self, value):
        """Change the unit for the field

        Parameters
        ----------
        value : str
            Units for the field.

        Examples
        --------
        Units for a displacement field.

        >>> from ansys.dpf import core as dpf
        >>> my_field = dpf.Field(10, dpf.natures.vector,dpf.locations.nodal)
        >>> my_field.unit = "m"
        >>> my_field.unit
        'm'

        """
        fielddef = self.field_definition
        fielddef.unit = value
        self.field_definition = fielddef

    @property
    def dimensionality(self):
        """Dimensionality represents the shape of the elementary data contained in the field.

        Returns
        -------
        dimensionality : :class:`ansys.dpf.core.dimensionality.Dimensionality`
            Nature and size of the elementary data.
        """
        if self.field_definition:
            return self.field_definition.dimensionality

    @dimensionality.setter
    def dimensionality(self, value):
        fielddef = self.field_definition
        fielddef.dimensionality = value
        self.field_definition = fielddef

    @property
    def name(self):
        """Name of the field."""
        request = field_pb2.GetRequest()
        request.field.CopyFrom(self._message)
        out = self._stub.GetFieldDefinition(request)
        return out.name

    def _set_field_definition(self, field_definition):
        """Set the field definition.

        Parameters
        ----------
        field_definition : :class"`ansys.dpf.core.field_definition.FieldDefinition`

        """
        request = field_pb2.UpdateFieldDefinitionRequest()
        request.field_def.CopyFrom(field_definition._messageDefinition)
        request.field.CopyFrom(self._message)
        self._stub.UpdateFieldDefinition(request)

    @property
    def field_definition(self):
        """Field information, including its location, unit, dimensionality, and shell layers.

        Returns
        -------
        :class:`ansys.dpf.core.field_definition.FieldDefinition`

        """
        return self._field_definition

    @field_definition.setter
    def field_definition(self, value):
        return self._set_field_definition(value)

    def _get_meshed_region(self):
        """Retrieve the meshed region.

        Returns
        -------
        :class:`ansys.dpf.core.meshed_region.MeshedRegion`

        """
        request = field_pb2.SupportRequest()
        request.field.CopyFrom(self._message)
        request.type = base_pb2.Type.Value("MESHED_REGION")
        try:
            message = self._stub.GetSupport(request)
            return meshed_region.MeshedRegion(mesh=message, server=self._server)
        except:
            raise RuntimeError(
                "The field's support is not a mesh. "
                "Try to retrieve the time frequency support."
            )

    def _get_time_freq_support(self):
        """Retrieve the time frequency support.

        Returns
        -------
        :class:`ansys.dpf.core.time_freq_support.TimeFreqSupport`

        """
        request = field_pb2.SupportRequest()
        request.field.CopyFrom(self._message)
        request.type = base_pb2.Type.Value("TIME_FREQ_SUPPORT")
        try:
            message = self._stub.GetSupport(request)
            return time_freq_support.TimeFreqSupport(
                time_freq_support=message, server=self._server
            )
        except:
            raise RuntimeError(
                "The field's support is not a timefreqsupport.  Try a mesh."
            )

    def _set_support(self, support, support_type: str):
        request = field_pb2.SetSupportRequest()
        request.field.CopyFrom(self._message)
        request.support.type = base_pb2.Type.Value(support_type)
        request.support.id = support._message.id
        self._stub.SetSupport(request)

    @property
    def time_freq_support(self):
        """Time frequency support of the field.

        Returns
        -------
        :class:`ansys.dpf.core.time_freq_support.TimeFreqSupport`

        """
        return self._get_time_freq_support()

    @time_freq_support.setter
    def time_freq_support(self, value):
        self._set_support(value, "TIME_FREQ_SUPPORT")

    @property
    def meshed_region(self):
        """Meshed region of the field.

        Return
        ------
        :class:`ansys.dpf.core.meshed_region.MeshedRegion`

        """
        return self._get_meshed_region()

    @meshed_region.setter
    def meshed_region(self, value):
        self._set_support(value, "MESHED_REGION")

    def __add__(self, field_b):
        """Add two fields.

        Returns
        -------
        :class:`ansys.dpf.core.operators.math.add.add`

        """
        from ansys.dpf.core import dpf_operator, operators

        if hasattr(operators, "math") and hasattr(operators.math, "add"):
            op = operators.math.add(server=self._server)
        else:
            op = dpf_operator.Operator("add", server=self._server)
        op.connect(0, self)
        op.connect(1, field_b)
        return op

    def __pow__(self, value):
        if value != 2:
            raise ValueError('Only the value "2" is suppported.')
        from ansys.dpf.core import dpf_operator, operators

        if hasattr(operators, "math") and hasattr(operators.math, "sqr"):
            op = operators.math.sqr(self, server=self._server)
        else:
            op = dpf_operator.Operator("sqr", server=self._server)
            op.connect(0, self)
        return op

    def __mul__(self, value):
        """Multiplies two fields.

        Returns
        -------
        :class:`ansys.dpf.core.operators.math.generalized_inner_product.generalized_inner_product`

        """
        from ansys.dpf.core import dpf_operator, operators

        if hasattr(operators, "math") and hasattr(
            operators.math, "generalized_inner_product"
        ):
            op = operators.math.generalized_inner_product(server=self._server)
        else:
            op = dpf_operator.Operator("generalized_inner_product", server=self._server)
        op.connect(0, self)
        op.connect(1, value)
        return op

    def __sub__(self, fields_b):
        """Subtract two fields.

        Returns
        -------
        :class:`ansys.dpf.core.operators.math.minus.minus`

        """
        from ansys.dpf.core import dpf_operator, operators

        if hasattr(operators, "math") and hasattr(operators.math, "minus"):
            op = operators.math.minus(server=self._server)
        else:
            op = dpf_operator.Operator("minus", server=self._server)
        op.connect(0, self)
        op.connect(1, fields_b)
        return op

    def _min_max(self):
        from ansys.dpf.core import dpf_operator

        op = dpf_operator.Operator("min_max", server=self._server)
        op.connect(0, self)
        return op

    def min(self):
        """Retrieve the component-wise minimum over this field.

        Returns
        -------
        min : Field
            Component-wise minimum field.
        """
        return self._min_max().get_output(0, types.field)

    def max(self):
        """Retrieve the component-wise maximum over this field.

        Returns
        -------
        max : Field
            Component-wise maximum field.
        """
        return self._min_max().get_output(1, types.field)

    def deep_copy(self, server=None):
        """Create a deep copy of the field's data on a given server.

        This method can be useful for passing data from one server instance to another.

        Parameters
        ----------
        server : :class:`ansys.dpf.core.server`, optional
        Server with the channel connected to the remote or local instance. The
        default is ``None``, in which case an attempt is made to use the global
        server.

        Returns
        -------
        field_copy : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> disp = model.results.displacement()
        >>> fields_container = disp.outputs.fields_container()
        >>> field = fields_container[0]
        >>> other_server = dpf.start_local_server(as_global=False)
        >>> deep_copy = field.deep_copy(server=other_server)

        """

        f = Field(
            nentities=len(self.scoping),
            location=self.location,
            nature=self.field_definition.dimensionality.nature,
            server=server,
        )
        f.scoping = self.scoping.deep_copy(server)
        f.data = self.data
        f.unit = self.unit
        f.location = self.location

        try:
            f._data_pointer = self._data_pointer
        except:
            pass
        try:
            f.meshed_region = self.meshed_region.deep_copy(server=server)
        except:
            pass
        try:
            f.time_freq_support = self.time_freq_support.deep_copy(server=server)
        except:
            pass

        return f


class _LocalField(_LocalFieldBase, Field):
    """Caches the internal data of a field so that it can be modified locally.

    A single update request is to the server when the local field is deleted.

    Parameters
    ----------
    field : Field
        Field to copy

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> import numpy as np
    >>> num_entities = 3
    >>> field_to_local = dpf.fields_factory.create_3d_vector_field(num_entities, location=dpf.locations.elemental_nodal)
    >>> with field_to_local.as_local_field() as f:
    ...     for i in range(1,num_entities+1):
    ...         f.append(np.array([[0.1*i,0.2*i, 0.3*i],[0.1*i,0.2*i, 0.3*i]]),i)
    ...         f.get_entity_data(i-1),[[0.1*i,0.2*i, 0.3*i],[0.1*i,0.2*i, 0.3*i]]
    (array([[0.1, 0.2, 0.3],
           [0.1, 0.2, 0.3]]), [[0.1, 0.2, 0.3], [0.1, 0.2, 0.3]])
    (array([[0.2, 0.4, 0.6],
           [0.2, 0.4, 0.6]]), [[0.2, 0.4, 0.6], [0.2, 0.4, 0.6]])
    (array([[0.3, 0.6, 0.9],
           [0.3, 0.6, 0.9]]), [[0.30000000000000004, 0.6000000000000001, 0.8999999999999999], [0.30000000000000004, 0.6000000000000001, 0.8999999999999999]])

    """  # noqa: E501

    def __init__(self, field):
        super().__init__(field)



"""
stress_ratio
============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "result" category
"""

class stress_ratio(Operator):
    """Read/compute element nodal stress ratio by calling the readers defined by the datasources. Regarding the requested location and the input mesh scoping, the result location can be Nodal/ElementalNodal/Elemental.

      available inputs:
        - time_scoping (Scoping, int, listfloat, Field, list) (optional)
        - mesh_scoping (ScopingsContainer, Scoping) (optional)
        - fields_container (FieldsContainer) (optional)
        - streams_container (StreamsContainer) (optional)
        - data_sources (DataSources)
        - bool_rotate_to_global (bool) (optional)
        - mesh (MeshedRegion, MeshesContainer) (optional)
        - requested_location (str) (optional)
        - read_cyclic (int) (optional)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.result.stress_ratio()

      >>> # Make input connections
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
      >>> my_mesh_scoping = dpf.ScopingsContainer()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
      >>> my_bool_rotate_to_global = bool()
      >>> op.inputs.bool_rotate_to_global.connect(my_bool_rotate_to_global)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)
      >>> my_read_cyclic = int()
      >>> op.inputs.read_cyclic.connect(my_read_cyclic)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.result.stress_ratio(time_scoping=my_time_scoping,mesh_scoping=my_mesh_scoping,data_sources=my_data_sources,requested_location=my_requested_location)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, time_scoping=None, mesh_scoping=None, data_sources=None, requested_location=None, config=None, server=None):
        super().__init__(name="ENL_SRAT", config = config, server = server)
        self._inputs = InputsStressRatio(self)
        self._outputs = OutputsStressRatio(self)
        if time_scoping !=None:
            self.inputs.time_scoping.connect(time_scoping)
        if mesh_scoping !=None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if data_sources !=None:
            self.inputs.data_sources.connect(data_sources)
        if requested_location !=None:
            self.inputs.requested_location.connect(requested_location)

    @staticmethod
    def _spec():
        spec = Specification(description="""Read/compute element nodal stress ratio by calling the readers defined by the datasources. Regarding the requested location and the input mesh scoping, the result location can be Nodal/ElementalNodal/Elemental.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "time_scoping", type_names=["scoping","int32","vector<int32>","double","field","vector<double>"], optional=True, document="""time/freq (use doubles or field), time/freq set ids (use ints or scoping) or time/freq step ids (use scoping with TimeFreq_steps location) requiered in output"""), 
                                 1 : PinSpecification(name = "mesh_scoping", type_names=["scopings_container","scoping"], optional=True, document="""nodes or elements scoping requiered in output. The scoping's location indicates whether nodes or elements are asked. Using scopings container enables to split the result fields container in domains"""), 
                                 2 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=True, document="""Fields container already allocated modified inplace"""), 
                                 3 : PinSpecification(name = "streams_container", type_names=["streams_container"], optional=True, document="""result file container allowed to be kept open to cache data"""), 
                                 4 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document="""result file path container, used if no streams are set"""), 
                                 5 : PinSpecification(name = "bool_rotate_to_global", type_names=["bool"], optional=True, document="""if true the field is rotated to global coordinate system (default true)"""), 
                                 7 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region","meshes_container"], optional=True, document="""prevents from reading the mesh in the result files"""), 
                                 9 : PinSpecification(name = "requested_location", type_names=["string"], optional=True, document="""requested location Nodal, Elemental or ElementalNodal"""), 
                                 14 : PinSpecification(name = "read_cyclic", type_names=["int32"], optional=True, document="""if 0 cyclic symmetry is ignored, if 1 cyclic sector is read, if 2 cyclic expansion is done, if 3 cyclic expansion is done and stages are merged (default is 1)""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "ENL_SRAT")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsStressRatio 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsStressRatio 
        """
        return super().outputs


#internal name: ENL_SRAT
#scripting name: stress_ratio
class InputsStressRatio(_Inputs):
    """Intermediate class used to connect user inputs to stress_ratio operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.stress_ratio()
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
      >>> my_mesh_scoping = dpf.ScopingsContainer()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
      >>> my_bool_rotate_to_global = bool()
      >>> op.inputs.bool_rotate_to_global.connect(my_bool_rotate_to_global)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)
      >>> my_read_cyclic = int()
      >>> op.inputs.read_cyclic.connect(my_read_cyclic)
    """
    def __init__(self, op: Operator):
        super().__init__(stress_ratio._spec().inputs, op)
        self._time_scoping = Input(stress_ratio._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._time_scoping)
        self._mesh_scoping = Input(stress_ratio._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh_scoping)
        self._fields_container = Input(stress_ratio._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._fields_container)
        self._streams_container = Input(stress_ratio._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._streams_container)
        self._data_sources = Input(stress_ratio._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._data_sources)
        self._bool_rotate_to_global = Input(stress_ratio._spec().input_pin(5), 5, op, -1) 
        self._inputs.append(self._bool_rotate_to_global)
        self._mesh = Input(stress_ratio._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._mesh)
        self._requested_location = Input(stress_ratio._spec().input_pin(9), 9, op, -1) 
        self._inputs.append(self._requested_location)
        self._read_cyclic = Input(stress_ratio._spec().input_pin(14), 14, op, -1) 
        self._inputs.append(self._read_cyclic)

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator

        - pindoc: time/freq (use doubles or field), time/freq set ids (use ints or scoping) or time/freq step ids (use scoping with TimeFreq_steps location) requiered in output

        Parameters
        ----------
        my_time_scoping : Scoping, int, list, float, Field, list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.stress_ratio()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> #or
        >>> op.inputs.time_scoping(my_time_scoping)

        """
        return self._time_scoping

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator

        - pindoc: nodes or elements scoping requiered in output. The scoping's location indicates whether nodes or elements are asked. Using scopings container enables to split the result fields container in domains

        Parameters
        ----------
        my_mesh_scoping : ScopingsContainer, Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.stress_ratio()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> #or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)

        """
        return self._mesh_scoping

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        - pindoc: Fields container already allocated modified inplace

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.stress_ratio()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator

        - pindoc: result file container allowed to be kept open to cache data

        Parameters
        ----------
        my_streams_container : StreamsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.stress_ratio()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> #or
        >>> op.inputs.streams_container(my_streams_container)

        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator

        - pindoc: result file path container, used if no streams are set

        Parameters
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.stress_ratio()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> #or
        >>> op.inputs.data_sources(my_data_sources)

        """
        return self._data_sources

    @property
    def bool_rotate_to_global(self):
        """Allows to connect bool_rotate_to_global input to the operator

        - pindoc: if true the field is rotated to global coordinate system (default true)

        Parameters
        ----------
        my_bool_rotate_to_global : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.stress_ratio()
        >>> op.inputs.bool_rotate_to_global.connect(my_bool_rotate_to_global)
        >>> #or
        >>> op.inputs.bool_rotate_to_global(my_bool_rotate_to_global)

        """
        return self._bool_rotate_to_global

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: prevents from reading the mesh in the result files

        Parameters
        ----------
        my_mesh : MeshedRegion, MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.stress_ratio()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator

        - pindoc: requested location Nodal, Elemental or ElementalNodal

        Parameters
        ----------
        my_requested_location : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.stress_ratio()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> #or
        >>> op.inputs.requested_location(my_requested_location)

        """
        return self._requested_location

    @property
    def read_cyclic(self):
        """Allows to connect read_cyclic input to the operator

        - pindoc: if 0 cyclic symmetry is ignored, if 1 cyclic sector is read, if 2 cyclic expansion is done, if 3 cyclic expansion is done and stages are merged (default is 1)

        Parameters
        ----------
        my_read_cyclic : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.stress_ratio()
        >>> op.inputs.read_cyclic.connect(my_read_cyclic)
        >>> #or
        >>> op.inputs.read_cyclic(my_read_cyclic)

        """
        return self._read_cyclic

class OutputsStressRatio(_Outputs):
    """Intermediate class used to get outputs from stress_ratio operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.stress_ratio()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(stress_ratio._spec().outputs, op)
        self._fields_container = Output(stress_ratio._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.stress_ratio()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
fft_multi_harmonic_minmax
=========================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Math plugin, from "math" category
"""

class fft_multi_harmonic_minmax(Operator):
    """Evaluate min max fields on multi harmonic solution. min and max fields are calculated based on evaluating a fft wrt rpms and using the gradient method for adaptive time steping

      available inputs:
        - fields_container (FieldsContainer)
        - rpm_scoping (Scoping) (optional)
        - fs_ratio (int) (optional)
        - num_subdivisions (int) (optional)
        - max_num_subdivisions (int) (optional)

      available outputs:
        - field_min (FieldsContainer)
        - field_max (FieldsContainer)
        - all_fields (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.math.fft_multi_harmonic_minmax()

      >>> # Make input connections
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_rpm_scoping = dpf.Scoping()
      >>> op.inputs.rpm_scoping.connect(my_rpm_scoping)
      >>> my_fs_ratio = int()
      >>> op.inputs.fs_ratio.connect(my_fs_ratio)
      >>> my_num_subdivisions = int()
      >>> op.inputs.num_subdivisions.connect(my_num_subdivisions)
      >>> my_max_num_subdivisions = int()
      >>> op.inputs.max_num_subdivisions.connect(my_max_num_subdivisions)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.math.fft_multi_harmonic_minmax(fields_container=my_fields_container,rpm_scoping=my_rpm_scoping,fs_ratio=my_fs_ratio,num_subdivisions=my_num_subdivisions,max_num_subdivisions=my_max_num_subdivisions)

      >>> # Get output data
      >>> result_field_min = op.outputs.field_min()
      >>> result_field_max = op.outputs.field_max()
      >>> result_all_fields = op.outputs.all_fields()"""
    def __init__(self, fields_container=None, rpm_scoping=None, fs_ratio=None, num_subdivisions=None, max_num_subdivisions=None, config=None, server=None):
        super().__init__(name="fft_multi_harmonic_minmax", config = config, server = server)
        self._inputs = InputsFftMultiHarmonicMinmax(self)
        self._outputs = OutputsFftMultiHarmonicMinmax(self)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if rpm_scoping !=None:
            self.inputs.rpm_scoping.connect(rpm_scoping)
        if fs_ratio !=None:
            self.inputs.fs_ratio.connect(fs_ratio)
        if num_subdivisions !=None:
            self.inputs.num_subdivisions.connect(num_subdivisions)
        if max_num_subdivisions !=None:
            self.inputs.max_num_subdivisions.connect(max_num_subdivisions)

    @staticmethod
    def _spec():
        spec = Specification(description="""Evaluate min max fields on multi harmonic solution. min and max fields are calculated based on evaluating a fft wrt rpms and using the gradient method for adaptive time steping""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "rpm_scoping", type_names=["scoping"], optional=True, document="""rpm scoping, by default the fft is evaluted using all the rpms"""), 
                                 2 : PinSpecification(name = "fs_ratio", type_names=["int32"], optional=True, document="""field or fields container with only one field is expected"""), 
                                 3 : PinSpecification(name = "num_subdivisions", type_names=["int32"], optional=True, document="""connect number subdivisions, used for uniform discretization"""), 
                                 4 : PinSpecification(name = "max_num_subdivisions", type_names=["int32"], optional=True, document="""connect max number subdivisions, used to avoid huge number of sudivisions""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field_min", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "field_max", type_names=["fields_container"], optional=False, document=""""""), 
                                 2 : PinSpecification(name = "all_fields", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "fft_multi_harmonic_minmax")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsFftMultiHarmonicMinmax 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsFftMultiHarmonicMinmax 
        """
        return super().outputs


#internal name: fft_multi_harmonic_minmax
#scripting name: fft_multi_harmonic_minmax
class InputsFftMultiHarmonicMinmax(_Inputs):
    """Intermediate class used to connect user inputs to fft_multi_harmonic_minmax operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.fft_multi_harmonic_minmax()
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_rpm_scoping = dpf.Scoping()
      >>> op.inputs.rpm_scoping.connect(my_rpm_scoping)
      >>> my_fs_ratio = int()
      >>> op.inputs.fs_ratio.connect(my_fs_ratio)
      >>> my_num_subdivisions = int()
      >>> op.inputs.num_subdivisions.connect(my_num_subdivisions)
      >>> my_max_num_subdivisions = int()
      >>> op.inputs.max_num_subdivisions.connect(my_max_num_subdivisions)
    """
    def __init__(self, op: Operator):
        super().__init__(fft_multi_harmonic_minmax._spec().inputs, op)
        self._fields_container = Input(fft_multi_harmonic_minmax._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_container)
        self._rpm_scoping = Input(fft_multi_harmonic_minmax._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._rpm_scoping)
        self._fs_ratio = Input(fft_multi_harmonic_minmax._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._fs_ratio)
        self._num_subdivisions = Input(fft_multi_harmonic_minmax._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._num_subdivisions)
        self._max_num_subdivisions = Input(fft_multi_harmonic_minmax._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._max_num_subdivisions)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_multi_harmonic_minmax()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def rpm_scoping(self):
        """Allows to connect rpm_scoping input to the operator

        - pindoc: rpm scoping, by default the fft is evaluted using all the rpms

        Parameters
        ----------
        my_rpm_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_multi_harmonic_minmax()
        >>> op.inputs.rpm_scoping.connect(my_rpm_scoping)
        >>> #or
        >>> op.inputs.rpm_scoping(my_rpm_scoping)

        """
        return self._rpm_scoping

    @property
    def fs_ratio(self):
        """Allows to connect fs_ratio input to the operator

        - pindoc: field or fields container with only one field is expected

        Parameters
        ----------
        my_fs_ratio : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_multi_harmonic_minmax()
        >>> op.inputs.fs_ratio.connect(my_fs_ratio)
        >>> #or
        >>> op.inputs.fs_ratio(my_fs_ratio)

        """
        return self._fs_ratio

    @property
    def num_subdivisions(self):
        """Allows to connect num_subdivisions input to the operator

        - pindoc: connect number subdivisions, used for uniform discretization

        Parameters
        ----------
        my_num_subdivisions : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_multi_harmonic_minmax()
        >>> op.inputs.num_subdivisions.connect(my_num_subdivisions)
        >>> #or
        >>> op.inputs.num_subdivisions(my_num_subdivisions)

        """
        return self._num_subdivisions

    @property
    def max_num_subdivisions(self):
        """Allows to connect max_num_subdivisions input to the operator

        - pindoc: connect max number subdivisions, used to avoid huge number of sudivisions

        Parameters
        ----------
        my_max_num_subdivisions : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_multi_harmonic_minmax()
        >>> op.inputs.max_num_subdivisions.connect(my_max_num_subdivisions)
        >>> #or
        >>> op.inputs.max_num_subdivisions(my_max_num_subdivisions)

        """
        return self._max_num_subdivisions

class OutputsFftMultiHarmonicMinmax(_Outputs):
    """Intermediate class used to get outputs from fft_multi_harmonic_minmax operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.fft_multi_harmonic_minmax()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field_min = op.outputs.field_min()
      >>> result_field_max = op.outputs.field_max()
      >>> result_all_fields = op.outputs.all_fields()
    """
    def __init__(self, op: Operator):
        super().__init__(fft_multi_harmonic_minmax._spec().outputs, op)
        self._field_min = Output(fft_multi_harmonic_minmax._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field_min)
        self._field_max = Output(fft_multi_harmonic_minmax._spec().output_pin(1), 1, op) 
        self._outputs.append(self._field_max)
        self._all_fields = Output(fft_multi_harmonic_minmax._spec().output_pin(2), 2, op) 
        self._outputs.append(self._all_fields)

    @property
    def field_min(self):
        """Allows to get field_min output of the operator


        Returns
        ----------
        my_field_min : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_multi_harmonic_minmax()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_min = op.outputs.field_min() 
        """
        return self._field_min

    @property
    def field_max(self):
        """Allows to get field_max output of the operator


        Returns
        ----------
        my_field_max : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_multi_harmonic_minmax()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_max = op.outputs.field_max() 
        """
        return self._field_max

    @property
    def all_fields(self):
        """Allows to get all_fields output of the operator


        Returns
        ----------
        my_all_fields : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_multi_harmonic_minmax()
        >>> # Connect inputs : op.inputs. ...
        >>> result_all_fields = op.outputs.all_fields() 
        """
        return self._all_fields




from ansys import dpf
import os


class PinSpecification:
    def __init__(self, name = None, type_names = None, optional = None, document = None, ellipsis = None):
        self.name = name
        self.type_names = type_names
        self.document = document
        self.optional = optional
        self.ellipsis = ellipsis
        

class Specification:
    def __init__(self, description = None, map_input_pin_spec = None, map_output_pin_spec = None):
        self.description = description
        self._map_input_pin_spec = map_input_pin_spec
        self._map_output_pin_spec = map_output_pin_spec
     
    @property
    def inputs(self):
        return self._map_input_pin_spec
        
    @property
    def outputs(self):
        return self._map_output_pin_spec
    
    def output_pin(self, pin_num):
        return self._map_output_pin_spec[pin_num]
    
    def input_pin(self, pin_num):
        return self._map_input_pin_spec[pin_num]
    
        


"""
enrich_materials
================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "logic" category
"""

class enrich_materials(Operator):
    """Take a MaterialContainer and a stream and enrich the MaterialContainer using stream data.

      available inputs:
        - MaterialContainer (Any)
        - streams (StreamsContainer, FieldsContainer)
        - streams_mapping ()

      available outputs:
        - MaterialContainer (bool)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.logic.enrich_materials()

      >>> # Make input connections
      >>> my_MaterialContainer = dpf.Any()
      >>> op.inputs.MaterialContainer.connect(my_MaterialContainer)
      >>> my_streams = dpf.StreamsContainer()
      >>> op.inputs.streams.connect(my_streams)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.logic.enrich_materials(MaterialContainer=my_MaterialContainer,streams=my_streams)

      >>> # Get output data
      >>> result_MaterialContainer = op.outputs.MaterialContainer()"""
    def __init__(self, MaterialContainer=None, streams=None, config=None, server=None):
        super().__init__(name="enrich_materials", config = config, server = server)
        self._inputs = InputsEnrichMaterials(self)
        self._outputs = OutputsEnrichMaterials(self)
        if MaterialContainer !=None:
            self.inputs.MaterialContainer.connect(MaterialContainer)
        if streams !=None:
            self.inputs.streams.connect(streams)

    @staticmethod
    def _spec():
        spec = Specification(description="""Take a MaterialContainer and a stream and enrich the MaterialContainer using stream data.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "MaterialContainer", type_names=["any"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "streams", type_names=["streams_container","fields_container"], optional=False, document=""""""), 
                                 2 : PinSpecification(name = "streams_mapping", type_names=[], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "MaterialContainer", type_names=["bool"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "enrich_materials")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsEnrichMaterials 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsEnrichMaterials 
        """
        return super().outputs


#internal name: enrich_materials
#scripting name: enrich_materials
class InputsEnrichMaterials(_Inputs):
    """Intermediate class used to connect user inputs to enrich_materials operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.logic.enrich_materials()
      >>> my_MaterialContainer = dpf.Any()
      >>> op.inputs.MaterialContainer.connect(my_MaterialContainer)
      >>> my_streams = dpf.StreamsContainer()
      >>> op.inputs.streams.connect(my_streams)
    """
    def __init__(self, op: Operator):
        super().__init__(enrich_materials._spec().inputs, op)
        self._MaterialContainer = Input(enrich_materials._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._MaterialContainer)
        self._streams = Input(enrich_materials._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._streams)

    @property
    def MaterialContainer(self):
        """Allows to connect MaterialContainer input to the operator

        Parameters
        ----------
        my_MaterialContainer : Any, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.enrich_materials()
        >>> op.inputs.MaterialContainer.connect(my_MaterialContainer)
        >>> #or
        >>> op.inputs.MaterialContainer(my_MaterialContainer)

        """
        return self._MaterialContainer

    @property
    def streams(self):
        """Allows to connect streams input to the operator

        Parameters
        ----------
        my_streams : StreamsContainer, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.enrich_materials()
        >>> op.inputs.streams.connect(my_streams)
        >>> #or
        >>> op.inputs.streams(my_streams)

        """
        return self._streams

class OutputsEnrichMaterials(_Outputs):
    """Intermediate class used to get outputs from enrich_materials operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.logic.enrich_materials()
      >>> # Connect inputs : op.inputs. ...
      >>> result_MaterialContainer = op.outputs.MaterialContainer()
    """
    def __init__(self, op: Operator):
        super().__init__(enrich_materials._spec().outputs, op)
        self._MaterialContainer = Output(enrich_materials._spec().output_pin(0), 0, op) 
        self._outputs.append(self._MaterialContainer)

    @property
    def MaterialContainer(self):
        """Allows to get MaterialContainer output of the operator


        Returns
        ----------
        my_MaterialContainer : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.enrich_materials()
        >>> # Connect inputs : op.inputs. ...
        >>> result_MaterialContainer = op.outputs.MaterialContainer() 
        """
        return self._MaterialContainer




# -*- coding: utf-8 -*-

"""
ScopingsContainer
=================
Contains classes associated to the DPF ScopingsContainer"""
from ansys import dpf
from ansys.dpf.core.collection import Collection
from ansys.dpf.core.common import types


class ScopingsContainer(Collection):
    """A class used to represent a ScopingsContainer which contains
    scopings split on a given space

    Parameters
    ----------
    scopings_container : ansys.grpc.dpf.collection_pb2.Collection or
                         ansys.dpf.core.ScopingsContainer, optional
        Create a scopings container from a Collection message or create
        a copy from an existing scopings container

    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the the global server.
    """

    def __init__(self, scopings_container=None, server=None):
        """Initialize the scoping with either optional scoping message,
        or by connecting to a stub.
        """
        if server is None:
            server = dpf.core._global_server()

        self._server = server
        self._stub = self._connect()

        Collection.__init__(
            self, types.scoping, collection=scopings_container, server=self._server
        )

    def get_scopings(self, label_space):
        """Returns the scopings at a requested label space

        Parameters
        ----------
        label_space_or_index : dict[str,int] , int
            Scopings corresponding to the filter (label space) in input, for example:
            ``{"elshape":1, "body":12}``

        Returns
        -------
        scopings : list[Scoping]
            scopings corresponding to the request
        """
        return super()._get_entries(label_space)

    def get_scoping(self, label_space_or_index):
        """Returns the scoping at a requested index or label space.
        Throws if the request returns several scoping

        Parameters
        ----------
        label_space_or_index : dict[str,int] , int
            Scopings corresponding to the filter (label space) in input, for example:
            ``{"elshape":1, "body":12}``
            or Index of the scoping.

        Returns
        -------
        scopings : Scoping
            scoping corresponding to the request
        """
        return super()._get_entry(label_space_or_index)

    def __getitem__(self, key):
        """Returns the scoping at a requested index

        Parameters
        ----------
        key : int
            the index

        Returns
        -------
        scoping : Scoping
            scoping corresponding to the request
        """
        return super().__getitem__(key)

    def add_scoping(self, label_space, scoping):
        """Update or add the scoping at a requested label space.

        Parameters
        ----------
        label_space : dict[str,int]
            label_space of the requested scopings, ex : {"elshape":1, "body":12}

        scoping : Scoping
            DPF scoping to add.
        """
        return super()._add_entry(label_space, scoping)

    def __str__(self):
        txt = "DPF Scopings Container with\n"
        txt += "\t%d scoping(s)\n" % len(self)
        txt += f"\tdefined on labels {self.labels} \n\n"
        return txt



from .field_to_fc import field_to_fc 
from .html_doc import html_doc 
from .unitary_field import unitary_field 
from .extract_field import extract_field 
from .bind_support import bind_support 
from .scalars_to_field import scalars_to_field 
from .change_location import change_location 
from .strain_from_voigt import strain_from_voigt 
from .set_property import set_property 
from .forward_field import forward_field 
from .forward_fields_container import forward_fields_container 
from .forward_meshes_container import forward_meshes_container 
from .forward import forward 
from .txt_file_to_dpf import txt_file_to_dpf 
from .bind_support_fc import bind_support_fc 
from .python_generator import python_generator 
from .change_shell_layers import change_shell_layers 



"""
AvailableResult
===============
"""

from warnings import warn
from ansys.grpc.dpf import available_result_pb2, base_pb2
from ansys.dpf.core.common import _remove_spaces


class AvailableResult:
    """Represents a result that can be requested via an operator.

    Parameters
    ----------
    availableresult : available_result_pb2.AvailableResult message.

    Examples
    --------
    Explore an available result from the model.

    >>> from ansys.dpf import core as dpf
    >>> from ansys.dpf.core import examples
    >>> transient = examples.download_transient_result()
    >>> model = dpf.Model(transient)
    >>> result_info = model.metadata.result_info
    >>> res = result_info.available_results[0]
    >>> res.name
    'displacement'
    >>> res.homogeneity
    'length'
    >>> res.dimensionality
    'vector'

    Create the operator of the given available result.

    >>> disp = model.results.displacement()

    """

    def __init__(self, availableresult):
        """Initialize the AvailableResult with an availableResult message."""
        self._message = availableresult

    def __str__(self):
        txt = (
            self.name
            + "\n"
            + 'Operator name: "%s"\n' % self.operator_name
            + "Number of components: %d\n" % self.n_components
            + "Dimensionality: %s\n" % self.dimensionality
            + "Homogeneity: %s\n" % self.homogeneity
        )
        if self.unit:
            txt += "Units: %s\n" % self.unit
        return txt

    @property
    def name(self):
        """Result operator."""
        if self.operator_name in _result_properties:
            return _result_properties[self.operator_name]["scripting_name"]
        else:
            return _remove_spaces(self._message.physicsname)

    @property
    def n_components(self):
        """Number of components of the result."""
        return self._message.ncomp

    @property
    def dimensionality(self):
        """Dimensionality nature of the result, such as a vector, scalar, or tensor."""
        return base_pb2.Nature.Name(self._message.dimensionality).lower()

    @property
    def homogeneity(self):
        """Homogeneity of the result."""
        try:
            homogeneity = self._message.homogeneity
            if homogeneity == 117:
                return available_result_pb2.Homogeneity.Name(
                    available_result_pb2.Homogeneity.DIMENSIONLESS
                ).lower()
            return available_result_pb2.Homogeneity.Name(homogeneity).lower()
        except ValueError as exception:
            warn(str(exception))
            return ""

    @property
    def unit(self):
        """Unit of the result."""
        return self._message.unit.lower()

    @property
    def operator_name(self):
        """Name of the corresponding operator."""
        return self._message.name

    @property
    def sub_results(self):
        """List of the subresult."""
        rep_sub_res = self._message.sub_res
        list = []
        for sub_res in rep_sub_res:
            try:
                int(sub_res.name)
                dict = {
                    "name": "principal" + sub_res.name,
                    "operator name": sub_res.op_name,
                    "description": sub_res.description,
                }
            except:
                dict = {
                    "name": sub_res.name,
                    "operator name": sub_res.op_name,
                    "description": sub_res.description,
                }
            list.append(dict)
        return list

    @property
    def native_location(self):
        """Native location of the result."""
        if self.operator_name in _result_properties:
            return _result_properties[self.operator_name]["location"]

    @property
    def native_scoping_location(self):
        """Native scoping location of the result."""
        loc = self.native_location
        if loc == "ElementalNodal":
            return "Elemental"
        else:
            return loc


_result_properties = {
    "S": {"location": "ElementalNodal", "scripting_name": "stress"},
    "ENF": {"location": "ElementalNodal", "scripting_name": "element_nodal_forces"},
    "EPEL": {"location": "ElementalNodal", "scripting_name": "elastic_strain"},
    "EPPL": {"location": "ElementalNodal", "scripting_name": "plastic_strain"},
    "ECR": {"location": "ElementalNodal", "scripting_name": "creep_strain"},
    "BFE": {"location": "ElementalNodal", "scripting_name": "structural_temperature"},
    "ETH": {"location": "ElementalNodal", "scripting_name": "thermal_strain"},
    "ETH_SWL": {"location": "ElementalNodal", "scripting_name": "swelling_strains"},
    "ENG_VOL": {"location": "Elemental", "scripting_name": "elemental_volume"},
    "ENG_SE": {"location": "Elemental", "scripting_name": "stiffness_matrix_energy"},
    "ENG_AHO": {
        "location": "Elemental",
        "scripting_name": "artificial_hourglass_energy",
    },
    "ENG_KE": {"location": "Elemental", "scripting_name": "kinetic_energy"},
    "ENG_CO": {"location": "Elemental", "scripting_name": "co_energy"},
    "ENG_INC": {"location": "Elemental", "scripting_name": "incremental_energy"},
    "ENG_TH": {"location": "Elemental", "scripting_name": "thermal_dissipation_energy"},
    "U": {"location": "Nodal", "scripting_name": "displacement"},
    "V": {"location": "Nodal", "scripting_name": "velocity"},
    "A": {"location": "Nodal", "scripting_name": "acceleration"},
    "RF": {"location": "Nodal", "scripting_name": "reaction_force"},
    "F": {"location": "Nodal", "scripting_name": "nodal_force"},
    "M": {"location": "Nodal", "scripting_name": "nodal_moment"},
    "TEMP": {"location": "Nodal", "scripting_name": "temperature"},
    "EF": {"location": "ElementalNodal", "scripting_name": "electric_field"},
    "VOLT": {"location": "Nodal", "scripting_name": "electric_potential"},
    "TF": {"location": "ElementalNodal", "scripting_name": "heat_flux"},
    "UTOT": {"location": "Nodal", "scripting_name": "raw_displacement"},
    "RFTOT": {"location": "Nodal", "scripting_name": "raw_reaction_force"},
}



from .nodal_fraction_fc import nodal_fraction_fc 
from .elemental_nodal_to_nodal_elemental_fc import elemental_nodal_to_nodal_elemental_fc 
from .elemental_difference import elemental_difference 
from .elemental_nodal_to_nodal import elemental_nodal_to_nodal 
from .elemental_difference_fc import elemental_difference_fc 
from .elemental_nodal_to_nodal_fc import elemental_nodal_to_nodal_fc 
from .elemental_to_nodal import elemental_to_nodal 
from .elemental_to_nodal_fc import elemental_to_nodal_fc 
from .nodal_difference import nodal_difference 
from .nodal_difference_fc import nodal_difference_fc 
from .elemental_fraction_fc import elemental_fraction_fc 
from .to_nodal import to_nodal 
from .to_nodal_fc import to_nodal_fc 
from .elemental_nodal_to_nodal_elemental import elemental_nodal_to_nodal_elemental 
from .extend_to_mid_nodes import extend_to_mid_nodes 
from .extend_to_mid_nodes_fc import extend_to_mid_nodes_fc 
from .elemental_mean import elemental_mean 
from .elemental_mean_fc import elemental_mean_fc 
from .to_elemental_fc import to_elemental_fc 
from .gauss_to_node_fc import gauss_to_node_fc 
from .nodal_to_elemental import nodal_to_elemental 
from .nodal_to_elemental_fc import nodal_to_elemental_fc 



from .min_max_by_time import min_max_by_time 
from .phase_of_max import phase_of_max 
from .time_of_max_by_entity import time_of_max_by_entity 
from .min_max_by_entity import min_max_by_entity 
from .min_max_over_time_by_entity import min_max_over_time_by_entity 
from .max_over_time_by_entity import max_over_time_by_entity 
from .min_over_time_by_entity import min_over_time_by_entity 
from .time_of_min_by_entity import time_of_min_by_entity 
from .max_over_phase import max_over_phase 
from .min_max import min_max 
from .min_max_fc import min_max_fc 
from .min_max_over_label_fc import min_max_over_label_fc 
from .min_by_component import min_by_component 
from .max_by_component import max_by_component 
from .min_max_fc_inc import min_max_fc_inc 
from .min_max_inc import min_max_inc 



"""
on_named_selection
==================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "scoping" category
"""

class on_named_selection(Operator):
    """provides a scoping at a given location based on a given named selection

      available inputs:
        - requested_location (str)
        - named_selection_name (str)
        - int_inclusive (int) (optional)
        - streams_container (StreamsContainer) (optional)
        - data_sources (DataSources)

      available outputs:
        - mesh_scoping (Scoping)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.scoping.on_named_selection()

      >>> # Make input connections
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)
      >>> my_named_selection_name = str()
      >>> op.inputs.named_selection_name.connect(my_named_selection_name)
      >>> my_int_inclusive = int()
      >>> op.inputs.int_inclusive.connect(my_int_inclusive)
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.scoping.on_named_selection(requested_location=my_requested_location,named_selection_name=my_named_selection_name,int_inclusive=my_int_inclusive,streams_container=my_streams_container,data_sources=my_data_sources)

      >>> # Get output data
      >>> result_mesh_scoping = op.outputs.mesh_scoping()"""
    def __init__(self, requested_location=None, named_selection_name=None, int_inclusive=None, streams_container=None, data_sources=None, config=None, server=None):
        super().__init__(name="scoping_provider_by_ns", config = config, server = server)
        self._inputs = InputsOnNamedSelection(self)
        self._outputs = OutputsOnNamedSelection(self)
        if requested_location !=None:
            self.inputs.requested_location.connect(requested_location)
        if named_selection_name !=None:
            self.inputs.named_selection_name.connect(named_selection_name)
        if int_inclusive !=None:
            self.inputs.int_inclusive.connect(int_inclusive)
        if streams_container !=None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources !=None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        spec = Specification(description="""provides a scoping at a given location based on a given named selection""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "requested_location", type_names=["string"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "named_selection_name", type_names=["string"], optional=False, document="""the string is expected to be in upper case"""), 
                                 2 : PinSpecification(name = "int_inclusive", type_names=["int32"], optional=True, document="""If element scoping is requested on a nodal named selection, if Inclusive == 1 then add all the elements adjacent to the nodes.If Inclusive == 0, only the elements which have all their nodes in the named selection are included"""), 
                                 3 : PinSpecification(name = "streams_container", type_names=["streams_container"], optional=True, document=""""""), 
                                 4 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "mesh_scoping", type_names=["scoping"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "scoping_provider_by_ns")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsOnNamedSelection 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsOnNamedSelection 
        """
        return super().outputs


#internal name: scoping_provider_by_ns
#scripting name: on_named_selection
class InputsOnNamedSelection(_Inputs):
    """Intermediate class used to connect user inputs to on_named_selection operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.scoping.on_named_selection()
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)
      >>> my_named_selection_name = str()
      >>> op.inputs.named_selection_name.connect(my_named_selection_name)
      >>> my_int_inclusive = int()
      >>> op.inputs.int_inclusive.connect(my_int_inclusive)
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
    """
    def __init__(self, op: Operator):
        super().__init__(on_named_selection._spec().inputs, op)
        self._requested_location = Input(on_named_selection._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._requested_location)
        self._named_selection_name = Input(on_named_selection._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._named_selection_name)
        self._int_inclusive = Input(on_named_selection._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._int_inclusive)
        self._streams_container = Input(on_named_selection._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._streams_container)
        self._data_sources = Input(on_named_selection._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._data_sources)

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator

        Parameters
        ----------
        my_requested_location : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.on_named_selection()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> #or
        >>> op.inputs.requested_location(my_requested_location)

        """
        return self._requested_location

    @property
    def named_selection_name(self):
        """Allows to connect named_selection_name input to the operator

        - pindoc: the string is expected to be in upper case

        Parameters
        ----------
        my_named_selection_name : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.on_named_selection()
        >>> op.inputs.named_selection_name.connect(my_named_selection_name)
        >>> #or
        >>> op.inputs.named_selection_name(my_named_selection_name)

        """
        return self._named_selection_name

    @property
    def int_inclusive(self):
        """Allows to connect int_inclusive input to the operator

        - pindoc: If element scoping is requested on a nodal named selection, if Inclusive == 1 then add all the elements adjacent to the nodes.If Inclusive == 0, only the elements which have all their nodes in the named selection are included

        Parameters
        ----------
        my_int_inclusive : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.on_named_selection()
        >>> op.inputs.int_inclusive.connect(my_int_inclusive)
        >>> #or
        >>> op.inputs.int_inclusive(my_int_inclusive)

        """
        return self._int_inclusive

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator

        Parameters
        ----------
        my_streams_container : StreamsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.on_named_selection()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> #or
        >>> op.inputs.streams_container(my_streams_container)

        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator

        Parameters
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.on_named_selection()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> #or
        >>> op.inputs.data_sources(my_data_sources)

        """
        return self._data_sources

class OutputsOnNamedSelection(_Outputs):
    """Intermediate class used to get outputs from on_named_selection operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.scoping.on_named_selection()
      >>> # Connect inputs : op.inputs. ...
      >>> result_mesh_scoping = op.outputs.mesh_scoping()
    """
    def __init__(self, op: Operator):
        super().__init__(on_named_selection._spec().outputs, op)
        self._mesh_scoping = Output(on_named_selection._spec().output_pin(0), 0, op) 
        self._outputs.append(self._mesh_scoping)

    @property
    def mesh_scoping(self):
        """Allows to get mesh_scoping output of the operator


        Returns
        ----------
        my_mesh_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.on_named_selection()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh_scoping = op.outputs.mesh_scoping() 
        """
        return self._mesh_scoping




"""
migrate_to_h5dpf
================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Hdf5 plugin, from "result" category
"""

class migrate_to_h5dpf(Operator):
    """Read mesh properties from the results files contained in the streams or data sources and make those properties available through a mesh selection manager in output.

      available inputs:
        - filename (str)
        - comma_separated_list_of_results (str) (optional)
        - all_time_sets (bool) (optional)
        - streams_container (StreamsContainer) (optional)
        - data_sources (DataSources) (optional)

      available outputs:
        - migrated_file (DataSources)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.result.migrate_to_h5dpf()

      >>> # Make input connections
      >>> my_filename = str()
      >>> op.inputs.filename.connect(my_filename)
      >>> my_comma_separated_list_of_results = str()
      >>> op.inputs.comma_separated_list_of_results.connect(my_comma_separated_list_of_results)
      >>> my_all_time_sets = bool()
      >>> op.inputs.all_time_sets.connect(my_all_time_sets)
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.result.migrate_to_h5dpf(filename=my_filename,comma_separated_list_of_results=my_comma_separated_list_of_results,all_time_sets=my_all_time_sets,streams_container=my_streams_container,data_sources=my_data_sources)

      >>> # Get output data
      >>> result_migrated_file = op.outputs.migrated_file()"""
    def __init__(self, filename=None, comma_separated_list_of_results=None, all_time_sets=None, streams_container=None, data_sources=None, config=None, server=None):
        super().__init__(name="hdf5::h5dpf::migrate_file", config = config, server = server)
        self._inputs = InputsMigrateToH5dpf(self)
        self._outputs = OutputsMigrateToH5dpf(self)
        if filename !=None:
            self.inputs.filename.connect(filename)
        if comma_separated_list_of_results !=None:
            self.inputs.comma_separated_list_of_results.connect(comma_separated_list_of_results)
        if all_time_sets !=None:
            self.inputs.all_time_sets.connect(all_time_sets)
        if streams_container !=None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources !=None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        spec = Specification(description="""Read mesh properties from the results files contained in the streams or data sources and make those properties available through a mesh selection manager in output.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "filename", type_names=["string"], optional=False, document="""filename of the migrated file"""), 
                                 1 : PinSpecification(name = "comma_separated_list_of_results", type_names=["string"], optional=True, document="""list of result (source operator names) that will be stored. If empty, all available results will be converted."""), 
                                 2 : PinSpecification(name = "all_time_sets", type_names=["bool"], optional=True, document="""default is true"""), 
                                 3 : PinSpecification(name = "streams_container", type_names=["streams_container"], optional=True, document="""streams (result file container) (optional)"""), 
                                 4 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=True, document="""if the stream is null then we need to get the file path from the data sources""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "migrated_file", type_names=["data_sources"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "hdf5::h5dpf::migrate_file")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMigrateToH5dpf 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsMigrateToH5dpf 
        """
        return super().outputs


#internal name: hdf5::h5dpf::migrate_file
#scripting name: migrate_to_h5dpf
class InputsMigrateToH5dpf(_Inputs):
    """Intermediate class used to connect user inputs to migrate_to_h5dpf operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.migrate_to_h5dpf()
      >>> my_filename = str()
      >>> op.inputs.filename.connect(my_filename)
      >>> my_comma_separated_list_of_results = str()
      >>> op.inputs.comma_separated_list_of_results.connect(my_comma_separated_list_of_results)
      >>> my_all_time_sets = bool()
      >>> op.inputs.all_time_sets.connect(my_all_time_sets)
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
    """
    def __init__(self, op: Operator):
        super().__init__(migrate_to_h5dpf._spec().inputs, op)
        self._filename = Input(migrate_to_h5dpf._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._filename)
        self._comma_separated_list_of_results = Input(migrate_to_h5dpf._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._comma_separated_list_of_results)
        self._all_time_sets = Input(migrate_to_h5dpf._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._all_time_sets)
        self._streams_container = Input(migrate_to_h5dpf._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._streams_container)
        self._data_sources = Input(migrate_to_h5dpf._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._data_sources)

    @property
    def filename(self):
        """Allows to connect filename input to the operator

        - pindoc: filename of the migrated file

        Parameters
        ----------
        my_filename : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.migrate_to_h5dpf()
        >>> op.inputs.filename.connect(my_filename)
        >>> #or
        >>> op.inputs.filename(my_filename)

        """
        return self._filename

    @property
    def comma_separated_list_of_results(self):
        """Allows to connect comma_separated_list_of_results input to the operator

        - pindoc: list of result (source operator names) that will be stored. If empty, all available results will be converted.

        Parameters
        ----------
        my_comma_separated_list_of_results : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.migrate_to_h5dpf()
        >>> op.inputs.comma_separated_list_of_results.connect(my_comma_separated_list_of_results)
        >>> #or
        >>> op.inputs.comma_separated_list_of_results(my_comma_separated_list_of_results)

        """
        return self._comma_separated_list_of_results

    @property
    def all_time_sets(self):
        """Allows to connect all_time_sets input to the operator

        - pindoc: default is true

        Parameters
        ----------
        my_all_time_sets : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.migrate_to_h5dpf()
        >>> op.inputs.all_time_sets.connect(my_all_time_sets)
        >>> #or
        >>> op.inputs.all_time_sets(my_all_time_sets)

        """
        return self._all_time_sets

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator

        - pindoc: streams (result file container) (optional)

        Parameters
        ----------
        my_streams_container : StreamsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.migrate_to_h5dpf()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> #or
        >>> op.inputs.streams_container(my_streams_container)

        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator

        - pindoc: if the stream is null then we need to get the file path from the data sources

        Parameters
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.migrate_to_h5dpf()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> #or
        >>> op.inputs.data_sources(my_data_sources)

        """
        return self._data_sources

class OutputsMigrateToH5dpf(_Outputs):
    """Intermediate class used to get outputs from migrate_to_h5dpf operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.migrate_to_h5dpf()
      >>> # Connect inputs : op.inputs. ...
      >>> result_migrated_file = op.outputs.migrated_file()
    """
    def __init__(self, op: Operator):
        super().__init__(migrate_to_h5dpf._spec().outputs, op)
        self._migrated_file = Output(migrate_to_h5dpf._spec().output_pin(0), 0, op) 
        self._outputs.append(self._migrated_file)

    @property
    def migrated_file(self):
        """Allows to get migrated_file output of the operator


        Returns
        ----------
        my_migrated_file : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.migrate_to_h5dpf()
        >>> # Connect inputs : op.inputs. ...
        >>> result_migrated_file = op.outputs.migrated_file() 
        """
        return self._migrated_file




"""
identical_property_fields
=========================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "logic" category
"""

class identical_property_fields(Operator):
    """Take two property fields and compare them.

      available inputs:
        - property_fieldA (MeshedRegion)
        - property_fieldB (MeshedRegion)

      available outputs:
        - are_identical (bool)
        - informations (str)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.logic.identical_property_fields()

      >>> # Make input connections
      >>> my_property_fieldA = dpf.MeshedRegion()
      >>> op.inputs.property_fieldA.connect(my_property_fieldA)
      >>> my_property_fieldB = dpf.MeshedRegion()
      >>> op.inputs.property_fieldB.connect(my_property_fieldB)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.logic.identical_property_fields(property_fieldA=my_property_fieldA,property_fieldB=my_property_fieldB)

      >>> # Get output data
      >>> result_are_identical = op.outputs.are_identical()
      >>> result_informations = op.outputs.informations()"""
    def __init__(self, property_fieldA=None, property_fieldB=None, config=None, server=None):
        super().__init__(name="compare::property_field", config = config, server = server)
        self._inputs = InputsIdenticalPropertyFields(self)
        self._outputs = OutputsIdenticalPropertyFields(self)
        if property_fieldA !=None:
            self.inputs.property_fieldA.connect(property_fieldA)
        if property_fieldB !=None:
            self.inputs.property_fieldB.connect(property_fieldB)

    @staticmethod
    def _spec():
        spec = Specification(description="""Take two property fields and compare them.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "property_fieldA", type_names=["abstract_meshed_region"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "property_fieldB", type_names=["abstract_meshed_region"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "are_identical", type_names=["bool"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "informations", type_names=["string"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "compare::property_field")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsIdenticalPropertyFields 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsIdenticalPropertyFields 
        """
        return super().outputs


#internal name: compare::property_field
#scripting name: identical_property_fields
class InputsIdenticalPropertyFields(_Inputs):
    """Intermediate class used to connect user inputs to identical_property_fields operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.logic.identical_property_fields()
      >>> my_property_fieldA = dpf.MeshedRegion()
      >>> op.inputs.property_fieldA.connect(my_property_fieldA)
      >>> my_property_fieldB = dpf.MeshedRegion()
      >>> op.inputs.property_fieldB.connect(my_property_fieldB)
    """
    def __init__(self, op: Operator):
        super().__init__(identical_property_fields._spec().inputs, op)
        self._property_fieldA = Input(identical_property_fields._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._property_fieldA)
        self._property_fieldB = Input(identical_property_fields._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._property_fieldB)

    @property
    def property_fieldA(self):
        """Allows to connect property_fieldA input to the operator

        Parameters
        ----------
        my_property_fieldA : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.identical_property_fields()
        >>> op.inputs.property_fieldA.connect(my_property_fieldA)
        >>> #or
        >>> op.inputs.property_fieldA(my_property_fieldA)

        """
        return self._property_fieldA

    @property
    def property_fieldB(self):
        """Allows to connect property_fieldB input to the operator

        Parameters
        ----------
        my_property_fieldB : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.identical_property_fields()
        >>> op.inputs.property_fieldB.connect(my_property_fieldB)
        >>> #or
        >>> op.inputs.property_fieldB(my_property_fieldB)

        """
        return self._property_fieldB

class OutputsIdenticalPropertyFields(_Outputs):
    """Intermediate class used to get outputs from identical_property_fields operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.logic.identical_property_fields()
      >>> # Connect inputs : op.inputs. ...
      >>> result_are_identical = op.outputs.are_identical()
      >>> result_informations = op.outputs.informations()
    """
    def __init__(self, op: Operator):
        super().__init__(identical_property_fields._spec().outputs, op)
        self._are_identical = Output(identical_property_fields._spec().output_pin(0), 0, op) 
        self._outputs.append(self._are_identical)
        self._informations = Output(identical_property_fields._spec().output_pin(1), 1, op) 
        self._outputs.append(self._informations)

    @property
    def are_identical(self):
        """Allows to get are_identical output of the operator


        Returns
        ----------
        my_are_identical : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.identical_property_fields()
        >>> # Connect inputs : op.inputs. ...
        >>> result_are_identical = op.outputs.are_identical() 
        """
        return self._are_identical

    @property
    def informations(self):
        """Allows to get informations output of the operator


        Returns
        ----------
        my_informations : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.identical_property_fields()
        >>> # Connect inputs : op.inputs. ...
        >>> result_informations = op.outputs.informations() 
        """
        return self._informations




"""
.. _ref_model:

Model
=====
Module contains the Model class to manage file result models.


"""

from ansys import dpf
from ansys.dpf.core import Operator
from ansys.dpf.core.common import types
from ansys.dpf.core.data_sources import DataSources
from ansys.dpf.core.results import Results
from grpc._channel import _InactiveRpcError


class Model:
    """Connects to a gRPC DPF server and allows access to a result using the DPF framework.

    Parameters
    ----------
    data_sources : str, dpf.core.DataSources
        Accepts either a :class:`dpf.core.DataSources` instance or the name of the
        result file to open. The default is ``None``.
    server : server.DPFServer, optional
        Server with the channel connected to the remote or local instance. The
        default is ``None``, in which case an attempt is made to use the global
        server.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> from ansys.dpf.core import examples
    >>> transient = examples.download_transient_result()
    >>> model = dpf.Model(transient)

    """

    def __init__(self, data_sources=None, server=None):
        """Initialize connection with DPF server."""

        if server is None:
            server = dpf.core._global_server()

        self._server = server
        self._metadata = Metadata(data_sources, self._server)
        self._results = Results(self)

    @property
    def metadata(self):
        """Model metadata.

        Includes:

        - ``data_sources``
        - ``meshed_region``
        - ``time_freq_support``
        - ``result_info``
        - ``mesh_provider``

        Returns
        -------
        metadata : Metadata

        Examples
        --------
        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = Model(transient)

        Get the meshed region of the model and extract the element
        numbers.

        >>> meshed_region = model.metadata.meshed_region
        >>> meshed_region.elements.scoping.ids[2]
        759

        Get the data sources of the model.

        >>> ds = model.metadata.data_sources

        Print the number of result sets.

        >>> tf = model.metadata.time_freq_support
        >>> tf.n_sets
        35

        Get the unit system used in the analysis.

        >>> rinfo = model.metadata.result_info
        >>> rinfo.unit_system
        'Metric (m, kg, N, s, V, A)'

        """
        return self._metadata

    @property
    def results(self):
        """Available results of the model.

        Organizes the results from DPF into accessible methods. All the available
        results are dynamically created depending on the model's class:`ansys.dpf.core.result_info`.

        Returns
        -------
        Results
            Available results of the model.

        Attributes
        ----------
        all types of results : Result
            Result provider helper wrapping all types of provider available for a
            given result file.

            Examples
            --------
            Create a stress result from the model and choose its time and mesh scopings.

            >>> from ansys.dpf import core as dpf
            >>> from ansys.dpf.core import examples
            >>> model = dpf.Model(examples.electric_therm)
            >>> v = model.results.electric_potential
            >>> rf = model.results.reaction_force
            >>> dissip = model.results.thermal_dissipation_energy

        Examples
        --------
        Extract the result object from a model.

        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.simple_bar)
        >>> results = model.results # printable object

        Access the displacement at all times.

        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = Model(transient)
        >>> displacements = model.results.displacement.on_all_time_freqs.eval()

        """
        return self._results

    def __connect_op__(self, op):
        """Connect the data sources or the streams to the operator."""
        if self.metadata._stream_provider is not None and hasattr(op.inputs, "streams"):
            op.inputs.streams.connect(self.metadata._stream_provider.outputs)
        elif self.metadata._data_sources is not None and hasattr(
            op.inputs, "data_sources"
        ):
            op.inputs.data_sources.connect(self.metadata._data_sources)

    def operator(self, name):
        """Operator associated with the data sources of this model.

        Parameters
        ----------
        name : str
            Operator name, which must be valid.

        Examples
        --------
        Create a displacement operator.

        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = Model(transient)
        >>> disp = model.operator('U')

        Create a sum operator.

        >>> sum = model.operator('accumulate')

        """
        op = Operator(name=name, server=self._server)
        self.__connect_op__(op)
        return op

    def __str__(self):
        txt = "DPF Model\n"
        txt += "-" * 30 + "\n"
        txt += str(self.results)
        txt += "-" * 30 + "\n"
        txt += str(self.metadata.meshed_region)
        txt += "\n" + "-" * 30 + "\n"
        txt += str(self.metadata.time_freq_support)
        return txt

    def plot(self, color="w", show_edges=True, **kwargs):
        """Plot the mesh of the model.

        Examples
        --------
        Plot the model using the default options.

        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = Model(transient)
        >>> model.plot()

        """
        self.metadata.meshed_region.grid.plot(
            color=color, show_edges=show_edges, **kwargs
        )


class Metadata:
    """Contains the metadata of a data source.

    Parameters
    ----------
    data_sources :

    server : server.DPFServer
        Server with the channel connected to the remote or local instance.

    """

    def __init__(self, data_sources, server):
        self._server = server
        self._set_data_sources(data_sources)
        self._meshed_region = None
        self.result_info = None
        self._stream_provider = None
        self._time_freq_support = None
        self._cache_streams_provider()
        self._cache_result_info()

    def _cache_result_info(self):
        """Store result information."""
        self.result_info = self._load_result_info()

    def _cache_streams_provider(self):
        """Create a stream provider and cache it."""
        from ansys.dpf.core import operators

        if hasattr(operators, "metadata") and hasattr(
            operators.metadata, "stream_provider"
        ):
            self._stream_provider = operators.metadata.streams_provider(
                data_sources=self._data_sources, server=self._server
            )
        else:
            self._stream_provider = Operator("stream_provider", server=self._server)
            self._stream_provider.inputs.connect(self._data_sources)

    @property
    def time_freq_support(self):
        """Time frequency support.

        Returns
        -------
        ansys.dpf.core.time_freq_support.TimeFreqSupport
            Time frequency support.

        Examples
        --------
        >>> from ansys.dpf.core import Model
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = Model(transient)

        Get the number of sets from the result file.

        >>> tf = model.metadata.time_freq_support
        >>> tf.n_sets
        35

        Get the time values for the active result.

        >>> tf.time_frequencies.data
        array([0.        , 0.019975  , 0.039975  , 0.059975  , 0.079975  ,
               0.099975  , 0.119975  , 0.139975  , 0.159975  , 0.179975  ,
               0.199975  , 0.218975  , 0.238975  , 0.258975  , 0.278975  ,
               0.298975  , 0.318975  , 0.338975  , 0.358975  , 0.378975  ,
               0.398975  , 0.417975  , 0.437975  , 0.457975  , 0.477975  ,
               0.497975  , 0.517975  , 0.53754972, 0.55725277, 0.57711786,
               0.59702054, 0.61694639, 0.63683347, 0.65673452, 0.67662783])

        """
        if self._time_freq_support is None:
            timeProvider = Operator("TimeFreqSupportProvider", server=self._server)
            timeProvider.inputs.connect(self._stream_provider.outputs)
            self._time_freq_support = timeProvider.get_output(
                0, types.time_freq_support
            )
        return self._time_freq_support

    @property
    def data_sources(self):
        """Data sources instance.

        This data source can be connected to other operators.

        Returns
        -------
        data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)

        Connect the model data sources to the 'U' operator.

        >>> ds = model.metadata.data_sources
        >>> op = dpf.operators.result.displacement()
        >>> op.inputs.data_sources.connect(ds)

        """
        return self._data_sources

    @property
    def streams_provider(self):
        """Streams provider operator connected to the data sources.

        This streams provider can be connected to other operators.

        Returns
        -------
        streams_provider : operators.metadata.streams_provider

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)

        Connect the model data sources to the ``U`` operator.

        >>> streams = model.metadata.streams_provider
        >>> op = dpf.operators.result.displacement()
        >>> op.inputs.streams_container.connect(streams)

        """
        return self._stream_provider

    def _set_data_sources(self, var_inp):
        if isinstance(var_inp, dpf.core.DataSources):
            self._data_sources = var_inp
        elif isinstance(var_inp, str):
            self._data_sources = DataSources(var_inp, server=self._server)
        else:
            self._data_sources = DataSources(server=self._server)
        self._cache_streams_provider()

    def _load_result_info(self):
        """Returns a result info object"""
        op = Operator("ResultInfoProvider", server=self._server)
        op.inputs.connect(self._stream_provider.outputs)
        try:
            result_info = op.get_output(0, types.result_info)
        except _InactiveRpcError as e:
            # give the user a more helpful error
            if "results file is not defined in the Data sources" in e.details():
                raise RuntimeError("Unable to open result file") from None
            else:
                raise e
        return result_info

    @property
    def meshed_region(self):
        """Meshed region instance.

        Returns
        -------
        mesh : :class:`ansys.dpf.core.meshed_region`
            Mesh
        """
        # NOTE: this uses the cached mesh and we might consider
        # changing this
        if self._meshed_region is None:
            self._meshed_region = self.mesh_provider.get_output(0, types.meshed_region)
            self._meshed_region._set_stream_provider(self._stream_provider)

        return self._meshed_region

    @property
    def mesh_provider(self):
        """Mesh provider operator.

        This operator reads a mesh from the result file. The underlying
        operator symbol is the class:`ansys.dpf.core.operators.mesh.mesh_provider`
        operator.

        Returns
        -------
        mesh_provider : class:`ansys.dpf.core.operators.mesh.mesh_provider`
            Mesh provider operator.

        """
        try:
            tmp = Operator("MeshSelectionManagerProvider", server=self._server)
            tmp.inputs.connect(self._stream_provider.outputs)
            tmp.run()
        except:
            pass
        mesh_provider = Operator("MeshProvider", server=self._server)
        mesh_provider.inputs.connect(self._stream_provider.outputs)
        return mesh_provider

    @property
    def available_named_selections(self):
        """List of available named selections.

        Returns
        -------
        named_selections : list str
        """
        return self.meshed_region.available_named_selections

    def named_selection(self, named_selection):
        """Scoping containing the list of nodes or elements in the named selection.

        Parameters
        ----------
        named_selection : str
            name of the named selection

        Returns
        -------
        named_selection : :class:`ansys.dpf.core.scoping`
        """
        return self.meshed_region.named_selection(named_selection)



"""
split_fields
============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "mesh" category
"""

class split_fields(Operator):
    """Split the input field or fields container based on the input mesh regions 

      available inputs:
        - field_or_fields_container (Field, FieldsContainer)
        - mesh_controller (MeshesContainer)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.mesh.split_fields()

      >>> # Make input connections
      >>> my_field_or_fields_container = dpf.Field()
      >>> op.inputs.field_or_fields_container.connect(my_field_or_fields_container)
      >>> my_mesh_controller = dpf.MeshesContainer()
      >>> op.inputs.mesh_controller.connect(my_mesh_controller)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.mesh.split_fields(field_or_fields_container=my_field_or_fields_container,mesh_controller=my_mesh_controller)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, field_or_fields_container=None, mesh_controller=None, config=None, server=None):
        super().__init__(name="split_fields", config = config, server = server)
        self._inputs = InputsSplitFields(self)
        self._outputs = OutputsSplitFields(self)
        if field_or_fields_container !=None:
            self.inputs.field_or_fields_container.connect(field_or_fields_container)
        if mesh_controller !=None:
            self.inputs.mesh_controller.connect(mesh_controller)

    @staticmethod
    def _spec():
        spec = Specification(description="""Split the input field or fields container based on the input mesh regions """,
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field_or_fields_container", type_names=["field","fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "mesh_controller", type_names=["meshes_container"], optional=False, document="""body meshes in the mesh controller cannot be mixed shell/solid""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "split_fields")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsSplitFields 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsSplitFields 
        """
        return super().outputs


#internal name: split_fields
#scripting name: split_fields
class InputsSplitFields(_Inputs):
    """Intermediate class used to connect user inputs to split_fields operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mesh.split_fields()
      >>> my_field_or_fields_container = dpf.Field()
      >>> op.inputs.field_or_fields_container.connect(my_field_or_fields_container)
      >>> my_mesh_controller = dpf.MeshesContainer()
      >>> op.inputs.mesh_controller.connect(my_mesh_controller)
    """
    def __init__(self, op: Operator):
        super().__init__(split_fields._spec().inputs, op)
        self._field_or_fields_container = Input(split_fields._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field_or_fields_container)
        self._mesh_controller = Input(split_fields._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh_controller)

    @property
    def field_or_fields_container(self):
        """Allows to connect field_or_fields_container input to the operator

        Parameters
        ----------
        my_field_or_fields_container : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.split_fields()
        >>> op.inputs.field_or_fields_container.connect(my_field_or_fields_container)
        >>> #or
        >>> op.inputs.field_or_fields_container(my_field_or_fields_container)

        """
        return self._field_or_fields_container

    @property
    def mesh_controller(self):
        """Allows to connect mesh_controller input to the operator

        - pindoc: body meshes in the mesh controller cannot be mixed shell/solid

        Parameters
        ----------
        my_mesh_controller : MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.split_fields()
        >>> op.inputs.mesh_controller.connect(my_mesh_controller)
        >>> #or
        >>> op.inputs.mesh_controller(my_mesh_controller)

        """
        return self._mesh_controller

class OutputsSplitFields(_Outputs):
    """Intermediate class used to get outputs from split_fields operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mesh.split_fields()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(split_fields._spec().outputs, op)
        self._fields_container = Output(split_fields._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.split_fields()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
fft_gradient_eval
=================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Math plugin, from "math" category
"""

class fft_gradient_eval(Operator):
    """Evaluate min max based on the fast fourier transform at a given field, using gradient method for adaptative time step.

      available inputs:
        - fields_container (FieldsContainer)
        - time_scoping (Scoping) (optional)
        - fs_ratio (int) (optional)

      available outputs:
        - coefficients (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.math.fft_gradient_eval()

      >>> # Make input connections
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
      >>> my_fs_ratio = int()
      >>> op.inputs.fs_ratio.connect(my_fs_ratio)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.math.fft_gradient_eval(fields_container=my_fields_container,time_scoping=my_time_scoping,fs_ratio=my_fs_ratio)

      >>> # Get output data
      >>> result_coefficients = op.outputs.coefficients()"""
    def __init__(self, fields_container=None, time_scoping=None, fs_ratio=None, config=None, server=None):
        super().__init__(name="fft_eval_gr", config = config, server = server)
        self._inputs = InputsFftGradientEval(self)
        self._outputs = OutputsFftGradientEval(self)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if time_scoping !=None:
            self.inputs.time_scoping.connect(time_scoping)
        if fs_ratio !=None:
            self.inputs.fs_ratio.connect(fs_ratio)

    @staticmethod
    def _spec():
        spec = Specification(description="""Evaluate min max based on the fast fourier transform at a given field, using gradient method for adaptative time step.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "time_scoping", type_names=["scoping"], optional=True, document="""if specified only the results at these set ids are used"""), 
                                 2 : PinSpecification(name = "fs_ratio", type_names=["int32"], optional=True, document="""default value = 20""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "coefficients", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "fft_eval_gr")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsFftGradientEval 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsFftGradientEval 
        """
        return super().outputs


#internal name: fft_eval_gr
#scripting name: fft_gradient_eval
class InputsFftGradientEval(_Inputs):
    """Intermediate class used to connect user inputs to fft_gradient_eval operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.fft_gradient_eval()
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
      >>> my_fs_ratio = int()
      >>> op.inputs.fs_ratio.connect(my_fs_ratio)
    """
    def __init__(self, op: Operator):
        super().__init__(fft_gradient_eval._spec().inputs, op)
        self._fields_container = Input(fft_gradient_eval._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_container)
        self._time_scoping = Input(fft_gradient_eval._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._time_scoping)
        self._fs_ratio = Input(fft_gradient_eval._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._fs_ratio)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_gradient_eval()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator

        - pindoc: if specified only the results at these set ids are used

        Parameters
        ----------
        my_time_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_gradient_eval()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> #or
        >>> op.inputs.time_scoping(my_time_scoping)

        """
        return self._time_scoping

    @property
    def fs_ratio(self):
        """Allows to connect fs_ratio input to the operator

        - pindoc: default value = 20

        Parameters
        ----------
        my_fs_ratio : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_gradient_eval()
        >>> op.inputs.fs_ratio.connect(my_fs_ratio)
        >>> #or
        >>> op.inputs.fs_ratio(my_fs_ratio)

        """
        return self._fs_ratio

class OutputsFftGradientEval(_Outputs):
    """Intermediate class used to get outputs from fft_gradient_eval operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.fft_gradient_eval()
      >>> # Connect inputs : op.inputs. ...
      >>> result_coefficients = op.outputs.coefficients()
    """
    def __init__(self, op: Operator):
        super().__init__(fft_gradient_eval._spec().outputs, op)
        self._coefficients = Output(fft_gradient_eval._spec().output_pin(0), 0, op) 
        self._outputs.append(self._coefficients)

    @property
    def coefficients(self):
        """Allows to get coefficients output of the operator


        Returns
        ----------
        my_coefficients : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.fft_gradient_eval()
        >>> # Connect inputs : op.inputs. ...
        >>> result_coefficients = op.outputs.coefficients() 
        """
        return self._coefficients




"""
included_fields
===============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "logic" category
"""

class included_fields(Operator):
    """Check if one field belongs to another.

      available inputs:
        - fieldA (Field)
        - fieldB (Field)
        - double_value (float)
        - double_tolerance (float) (optional)

      available outputs:
        - included (bool)
        - message (str)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.logic.included_fields()

      >>> # Make input connections
      >>> my_fieldA = dpf.Field()
      >>> op.inputs.fieldA.connect(my_fieldA)
      >>> my_fieldB = dpf.Field()
      >>> op.inputs.fieldB.connect(my_fieldB)
      >>> my_double_value = float()
      >>> op.inputs.double_value.connect(my_double_value)
      >>> my_double_tolerance = float()
      >>> op.inputs.double_tolerance.connect(my_double_tolerance)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.logic.included_fields(fieldA=my_fieldA,fieldB=my_fieldB,double_value=my_double_value,double_tolerance=my_double_tolerance)

      >>> # Get output data
      >>> result_included = op.outputs.included()
      >>> result_message = op.outputs.message()"""
    def __init__(self, fieldA=None, fieldB=None, double_value=None, double_tolerance=None, config=None, server=None):
        super().__init__(name="Are_fields_included", config = config, server = server)
        self._inputs = InputsIncludedFields(self)
        self._outputs = OutputsIncludedFields(self)
        if fieldA !=None:
            self.inputs.fieldA.connect(fieldA)
        if fieldB !=None:
            self.inputs.fieldB.connect(fieldB)
        if double_value !=None:
            self.inputs.double_value.connect(double_value)
        if double_tolerance !=None:
            self.inputs.double_tolerance.connect(double_tolerance)

    @staticmethod
    def _spec():
        spec = Specification(description="""Check if one field belongs to another.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fieldA", type_names=["field"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "fieldB", type_names=["field"], optional=False, document=""""""), 
                                 2 : PinSpecification(name = "double_value", type_names=["double"], optional=False, document="""Double positive small value. Smallest value which will be considered during the comparison step: all the abs(values) in field less than this value is considered as null, (default value:1.0e-14)."""), 
                                 3 : PinSpecification(name = "double_tolerance", type_names=["double"], optional=True, document="""Double relative tolerance. Maximum tolerance gap between to compared values: values within relative tolerance are considered identical (v1-v2)/v2 < relativeTol (default is 0.001).""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "included", type_names=["bool"], optional=False, document="""bool (true if belongs...)"""), 
                                 1 : PinSpecification(name = "message", type_names=["string"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "Are_fields_included")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsIncludedFields 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsIncludedFields 
        """
        return super().outputs


#internal name: Are_fields_included
#scripting name: included_fields
class InputsIncludedFields(_Inputs):
    """Intermediate class used to connect user inputs to included_fields operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.logic.included_fields()
      >>> my_fieldA = dpf.Field()
      >>> op.inputs.fieldA.connect(my_fieldA)
      >>> my_fieldB = dpf.Field()
      >>> op.inputs.fieldB.connect(my_fieldB)
      >>> my_double_value = float()
      >>> op.inputs.double_value.connect(my_double_value)
      >>> my_double_tolerance = float()
      >>> op.inputs.double_tolerance.connect(my_double_tolerance)
    """
    def __init__(self, op: Operator):
        super().__init__(included_fields._spec().inputs, op)
        self._fieldA = Input(included_fields._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fieldA)
        self._fieldB = Input(included_fields._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._fieldB)
        self._double_value = Input(included_fields._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._double_value)
        self._double_tolerance = Input(included_fields._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._double_tolerance)

    @property
    def fieldA(self):
        """Allows to connect fieldA input to the operator

        Parameters
        ----------
        my_fieldA : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.included_fields()
        >>> op.inputs.fieldA.connect(my_fieldA)
        >>> #or
        >>> op.inputs.fieldA(my_fieldA)

        """
        return self._fieldA

    @property
    def fieldB(self):
        """Allows to connect fieldB input to the operator

        Parameters
        ----------
        my_fieldB : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.included_fields()
        >>> op.inputs.fieldB.connect(my_fieldB)
        >>> #or
        >>> op.inputs.fieldB(my_fieldB)

        """
        return self._fieldB

    @property
    def double_value(self):
        """Allows to connect double_value input to the operator

        - pindoc: Double positive small value. Smallest value which will be considered during the comparison step: all the abs(values) in field less than this value is considered as null, (default value:1.0e-14).

        Parameters
        ----------
        my_double_value : float, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.included_fields()
        >>> op.inputs.double_value.connect(my_double_value)
        >>> #or
        >>> op.inputs.double_value(my_double_value)

        """
        return self._double_value

    @property
    def double_tolerance(self):
        """Allows to connect double_tolerance input to the operator

        - pindoc: Double relative tolerance. Maximum tolerance gap between to compared values: values within relative tolerance are considered identical (v1-v2)/v2 < relativeTol (default is 0.001).

        Parameters
        ----------
        my_double_tolerance : float, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.included_fields()
        >>> op.inputs.double_tolerance.connect(my_double_tolerance)
        >>> #or
        >>> op.inputs.double_tolerance(my_double_tolerance)

        """
        return self._double_tolerance

class OutputsIncludedFields(_Outputs):
    """Intermediate class used to get outputs from included_fields operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.logic.included_fields()
      >>> # Connect inputs : op.inputs. ...
      >>> result_included = op.outputs.included()
      >>> result_message = op.outputs.message()
    """
    def __init__(self, op: Operator):
        super().__init__(included_fields._spec().outputs, op)
        self._included = Output(included_fields._spec().output_pin(0), 0, op) 
        self._outputs.append(self._included)
        self._message = Output(included_fields._spec().output_pin(1), 1, op) 
        self._outputs.append(self._message)

    @property
    def included(self):
        """Allows to get included output of the operator


        - pindoc: bool (true if belongs...)

        Returns
        ----------
        my_included : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.included_fields()
        >>> # Connect inputs : op.inputs. ...
        >>> result_included = op.outputs.included() 
        """
        return self._included

    @property
    def message(self):
        """Allows to get message output of the operator


        Returns
        ----------
        my_message : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.included_fields()
        >>> # Connect inputs : op.inputs. ...
        >>> result_message = op.outputs.message() 
        """
        return self._message

